@@ -18,7 +18,7 @@ import shutil
 import stat
 import subprocess
 import sys
-from typing import Generator, List, Optional
+from typing import List, Optional
 
 # logging output stream, setup in main()
 log = None
@@ -63,24 +63,17 @@ def upload(version: str, path: Path, name: Optional[str] = None) -> str:
 
   return target
 
-def checksum(path: Path) -> Generator[Path, None, None]:
-  """generate checksum(s) for the given file"""
+def checksum(path: Path) -> Path:
+  """generate checksum for the given file"""
 
   assert path.exists()
 
-  log.info(f"MD5 summing {path}")
-  check = Path(f"{path}.md5")
-  with open(check, "wt") as f:
-    with open(path, "rb") as data:
-      f.write(f"{hashlib.md5(data.read()).hexdigest()}  {path}\n")
-  yield check
-
   log.info(f"SHA256 summing {path}")
   check = Path(f"{path}.sha256")
-  with open(check, "wt") as f:
+  with open(check, "wt", encoding="utf-8") as f:
     with open(path, "rb") as data:
       f.write(f"{hashlib.sha256(data.read()).hexdigest()}  {path}\n")
-  yield check
+  return check
 
 def is_macos_artifact(path: Path) -> bool:
   """is this a deployment artifact for macOS?"""
@@ -90,6 +83,14 @@ def is_windows_artifact(path: Path) -> bool:
   """is this a deployment artifact for Windows?"""
   return re.search(r"\bwindows\b", str(path)) is not None
 
+def get_format(path: Path) -> str:
+  """a human readable description of the format of this file"""
+  if path.suffix[1:].lower() == "exe":
+    return "EXE installer"
+  if path.suffix[1:].lower() == "zip":
+    return "ZIP installer"
+  return path.suffix[1:].lower()
+
 def main(args: List[str]) -> int: # pylint: disable=missing-function-docstring
 
   # setup logging to print to stderr
@@ -119,7 +120,7 @@ def main(args: List[str]) -> int: # pylint: disable=missing-function-docstring
   # echo some useful things for debugging
   log.info(f"os.uname(): {os.uname()}")
   if Path("/etc/os-release").exists():
-    with open("/etc/os-release") as f:
+    with open("/etc/os-release", "rt", encoding="utf-8") as f:
       log.info("/etc/os-release:")
       for i, line in enumerate(f, 1):
         log.info(f" {i}: {line[:-1]}")
@@ -132,7 +133,7 @@ def main(args: List[str]) -> int: # pylint: disable=missing-function-docstring
 
   # retrieve version name left by prior CI tasks
   log.info("reading GRAPHVIZ_VERSION")
-  with open("GRAPHVIZ_VERSION") as f:
+  with open("GRAPHVIZ_VERSION", "rt", encoding="utf-8") as f:
     gv_version = f.read().strip()
   log.info(f"GRAPHVIZ_VERSION == {gv_version}")
 
@@ -170,17 +171,17 @@ def main(args: List[str]) -> int: # pylint: disable=missing-function-docstring
       log.error(f"source {tarball} not found")
       return -1
 
-    # accrue the source tarball and accompanying checksum(s)
+    # accrue the source tarball and accompanying checksum
     url = upload(package_version, tarball)
     assets.append(url)
     webentry = {
-      "format": tarball.suffix[1:],
+      "format": get_format(tarball),
       "url": url
     }
-    for check in checksum(tarball):
-      url = upload(package_version, check)
-      assets.append(url)
-      webentry[check.suffix[1:]] = url
+    check = checksum(tarball)
+    url = upload(package_version, check)
+    assets.append(url)
+    webentry[check.suffix[1:]] = url
 
     webdata["sources"].append(webentry)
 
@@ -194,13 +195,36 @@ def main(args: List[str]) -> int: # pylint: disable=missing-function-docstring
       # fixup permissions, o-rwx g-wx
       os.chmod(path, mode & ~stat.S_IRWXO & ~stat.S_IWGRP & ~stat.S_IXGRP)
 
-      assets.append(upload(package_version, path, str(path)[len("Packages/"):]))
+      url = upload(package_version, path, str(path)[len("Packages/"):])
+      assets.append(url)
+
+      webentry = {
+        "format": get_format(path),
+        "url": url,
+      }
+      if "win32" in str(path):
+        webentry["bits"] = 32
+      elif "win64" in str(path):
+        webentry["bits"] = 64
 
       # if this is a standalone Windows or macOS package, also provide
       # checksum(s)
       if is_macos_artifact(path) or is_windows_artifact(path):
-        for c in checksum(path):
-          assets.append(upload(package_version, c, str(c)[len("Packages/"):]))
+        c = checksum(path)
+        url = upload(package_version, c, str(c)[len("Packages/"):])
+        assets.append(url)
+        webentry[c.suffix[1:]] = url
+
+      # only expose a subset of the Windows artifacts
+      if "windows_10_cmake_Release" in str(path) or \
+         "windows_10_msbuild_Release" in str(path):
+        webdata["windows"].append(webentry)
+
+  # various release pages truncate the viewable artifacts to 100 or even 50
+  if not options.force and len(assets) > 50:
+    log.error(f"upload has {len(assets)} assets, which will result in some of "
+              "them being unviewable in web page lists")
+    return -1
 
   # we only create Gitlab releases for stable version numbers
   if not options.force:
@@ -223,7 +247,7 @@ def main(args: List[str]) -> int: # pylint: disable=missing-function-docstring
 
   # output JSON data for the website
   log.info(f"dumping {webdata} to graphviz-{options.version}.json")
-  with open(f"graphviz-{options.version}.json", "wt") as f:
+  with open(f"graphviz-{options.version}.json", "wt", encoding="utf-8") as f:
     json.dump(webdata, f, indent=2)
 
   return 0
