@@ -8,10 +8,6 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 /* Lefteris Koutsofios - AT&T Labs Research */
 
 #ifndef _COMMON_H
@@ -36,6 +32,7 @@ extern "C" {
 #define FEATURE_X11 1
 #endif
 
+#include <stdbool.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
@@ -91,7 +88,7 @@ extern fd_set inputfds;
 
 int init (char *);
 void term (void);
-char *buildpath (char *, int);
+char *buildpath (char *, bool);
 char *buildcommand (char *, char *, int, int, char *);
 void warning (char *, int, char *, char *, ...);
 
@@ -106,7 +103,3 @@ NORETURN void panic2 (char *, int, char *, char *, ...);
 #undef NORETURN
 
 #endif /* _COMMON_H */
-
-#ifdef __cplusplus
-}
-#endif
