@@ -16,17 +16,13 @@
 #include <xdot/xdot.h>
 #include <cgraph/cgraph.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
     void init_viewport(ViewInfo * view);
     void set_viewport_settings_from_template(ViewInfo * view, Agraph_t *);
     void switch_graph(int);
-    void refreshViewport(int doClear);
+    void refreshViewport(void);
     int add_graph_to_viewport_from_file(char *fileName);
     int add_graph_to_viewport(Agraph_t * graph, char *);
-    int close_graph(ViewInfo * view, int graphid);
+    int close_graph(ViewInfo * view);
     int save_graph(void);
     int save_graph_with_file_name(Agraph_t * graph, char *fileName);
     int save_as_graph(void);
@@ -43,7 +39,3 @@ extern "C" {
     /* helper functions */
     extern char *get_attribute_value(char *attr, ViewInfo * view,
 				     Agraph_t * g);
-
-#ifdef __cplusplus
-}				/* end extern "C" */
-#endif
