@@ -17,11 +17,43 @@
 #include "config.h"
 
 #include <ctype.h>
+#include <stdbool.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <cgraph/cgraph.h>
+#include <cgraph/likely.h>
+#include <cgraph/unreachable.h>
+#include <cgraph/exit.h>
 
-#define N_NEW(n,t)       calloc((n),sizeof(t))
-#define NEW(t)           malloc(sizeof(t))
+static void *xmalloc(size_t size) {
+  void *p = malloc(size);
+  if (UNLIKELY(size > 0 && p == NULL)) {
+    fprintf(stderr, "ccomps: out of memory\n");
+    graphviz_exit(EXIT_FAILURE);
+  }
+  return p;
+}
+
+static void *xcalloc(size_t count, size_t size) {
+  void *p = calloc(count, size);
+  if (UNLIKELY(count > 0 && size > 0 && p == NULL)) {
+    fprintf(stderr, "ccomps: out of memory\n");
+    graphviz_exit(EXIT_FAILURE);
+  }
+  return p;
+}
+
+static void *xrealloc(void *ptr, size_t size) {
+  void *p = realloc(ptr, size);
+  if (UNLIKELY(size > 0 && p == NULL)) {
+    fprintf(stderr, "ccomps: out of memory\n");
+    graphviz_exit(EXIT_FAILURE);
+  }
+  return p;
+}
+
+#define N_NEW(n,t)       xcalloc((n),sizeof(t))
+#define NEW(t)           xmalloc(sizeof(t))
 
 typedef struct {
     Agrec_t h;
@@ -98,19 +130,18 @@ If no files are specified, stdin is used\n";
 static void usage(int v)
 {
     printf("%s",useString);
-    exit(v);
+    graphviz_exit(v);
 }
 
 static void split(char *name)
 {
     char *sfx = 0;
-    int size;
 
     sfx = strrchr(name, '.');
     if (sfx) {
 	suffix = sfx + 1;
-	size = sfx - name;
-	path = malloc(size + 1);
+	size_t size = (size_t)(sfx - name);
+	path = xmalloc(size + 1);
 	strncpy(path, name, size);
 	*(path + size) = '\0';
     } else {
@@ -159,12 +190,12 @@ static void init(int argc, char *argv[])
 		char *p = optarg + 1;
 		if (*optarg == '#') x_mode = BY_INDEX;
 		else x_mode = BY_SIZE;
-		if (isdigit(*p)) {
+		if (isdigit((int)*p)) {
 		    x_index = (int)strtol (p, &endp, 10);
 		    printMode = EXTRACT;
 		    if (*endp == '-') {
 			p = endp + 1;
-			if (isdigit(*p)) {
+			if (isdigit((int)*p)) {
 			    x_final = atoi (p);
 			    if (x_final < x_index) {
 				printMode = INTERNAL;
@@ -210,6 +241,8 @@ static void init(int argc, char *argv[])
                 usage(1);
 	    }
 	    break;
+	default:
+	    UNREACHABLE();
 	}
     }
     argv += optind;
@@ -266,17 +299,9 @@ static void push(Agnode_t * np)
     if (Stk.curp == Stk.curblk->endp) {
 	if (Stk.curblk->next == NULL) {
 	    blk_t *bp = NEW(blk_t);
-	    if (bp == 0) {
-		fprintf(stderr, "gc: Out of memory\n");
-		exit(1);
-	    }
 	    bp->prev = Stk.curblk;
 	    bp->next = NULL;
 	    bp->data = N_NEW(BIGBUF, Agnode_t *);
-	    if (bp->data == 0) {
-		fprintf(stderr, "%s: Out of memory\n", Cmd);
-		exit(1);
-	    }
 	    bp->endp = bp->data + BIGBUF;
 	    Stk.curblk->next = bp;
 	}
@@ -350,7 +375,7 @@ static char *getName(void)
 	name = outfile;
     else {
 	if (!buf)
-	    buf = malloc(strlen(outfile) + 20);	/* enough to handle '_number' */
+	    buf = xmalloc(strlen(outfile) + 20);	/* enough to handle '_number' */
 	if (suffix)
 	    sprintf(buf, "%s_%d.%s", path, sufcnt, suffix);
 	else
@@ -386,18 +411,14 @@ static void gwrite(Agraph_t * g)
  * Return pointer to buffer containing at least n bytes.
  * Non-reentrant.
  */
-static char *getBuf(int n)
+static char *getBuf(size_t n)
 {
-    static int len = 0;
+    static size_t len = 0;
     static char *buf = 0;
-    int sz;
 
     if (n > len) {
-	sz = n + 100;
-	if (len == 0)
-	    buf = malloc(sz);
-	else
-	    buf = realloc(buf, sz);
+	size_t sz = n + 100;
+	buf = xrealloc(buf, sz);
 	len = sz;
     }
     return buf;
@@ -479,7 +500,7 @@ static void deriveClusters(Agraph_t* dg, Agraph_t * g)
     for (subg = agfstsubg(g); subg; subg = agnxtsubg(subg)) {
 	if (!strncmp(agnameof(subg), "cluster", 7)) {
 	    dn = agnode(dg, agnameof(subg), 1);
-	    agbindrec (dn, "nodeinfo", sizeof(Agnodeinfo_t), TRUE);
+	    agbindrec (dn, "nodeinfo", sizeof(Agnodeinfo_t), true);
 	    ND_ptr(dn) = (Agobj_t*)subg;
 	    for (n = agfstnode(subg); n; n = agnxtnode(subg, n)) {
 		if (ND_ptr(n)) {
@@ -514,7 +535,7 @@ static Agraph_t *deriveGraph(Agraph_t * g)
 	if (ND_dn(n))
 	    continue;
 	dn = agnode(dg, agnameof(n), 1);
-	agbindrec (dn, "nodeinfo", sizeof(Agnodeinfo_t), TRUE);
+	agbindrec (dn, "nodeinfo", sizeof(Agnodeinfo_t), true);
 	ND_ptr(dn) = (Agobj_t*)n;
 	ND_ptr(n) = (Agobj_t*)dn;
     }
@@ -858,7 +879,7 @@ static int process(Agraph_t * g, char* graphName)
 
 static Agraph_t *gread(FILE * fp)
 {
-    return agread(fp, (Agdisc_t *) 0);
+    return agread(fp, NULL);
 }
 
 /* chkGraphName:
@@ -876,14 +897,13 @@ static char*
 chkGraphName (Agraph_t* g)
 {
     static char* buf = NULL;
-    static int buflen = 0;
+    static size_t buflen = 0;
     char* s = agnameof(g);
-    int len;
 
     if (*s != '%') return s;
-    len = strlen(s) + 2;   /* plus '\0' and '_' */
+    size_t len = strlen(s) + 2;   /* plus '\0' and '_' */
     if (len > buflen) {
-	buf = realloc (buf, len);
+	buf = xrealloc (buf, len);
 	buflen = len;
     }
     buf[0] = '_';
@@ -905,5 +925,5 @@ int main(int argc, char *argv[])
 	agclose(g);
     }
 
-    return r;
+    graphviz_exit(r);
 }
