@@ -11,6 +11,7 @@
 
 #include    "convert.h"
 #include    <cgraph/agxbuf.h>
+#include    <cgraph/exit.h>
 #include    <getopt.h>
 #ifdef HAVE_EXPAT
 #include    <expat.h>
@@ -71,7 +72,7 @@ static void popString(slist ** stk)
     slist *sp = *stk;
     if (!sp) {
 	fprintf(stderr, "PANIC: graphml2gv: empty element stack\n");
-	exit(1);
+	graphviz_exit(1);
     }
     *stk = sp->next;
     free(sp);
@@ -81,7 +82,7 @@ static char *topString(slist * stk)
 {
     if (!stk) {
 	fprintf(stderr, "PANIC: graphml2gv: empty element stack\n");
-	exit(1);
+	graphviz_exit(1);
     }
     return stk->buf;
 }
@@ -141,15 +142,11 @@ static void free_nitem(Dt_t * d, namev_t * np, Dtdisc_t * disc)
 }
 
 static Dtdisc_t nameDisc = {
-    offsetof(namev_t, name),
-    -1,
-    offsetof(namev_t, link),
-    (Dtmake_f) make_nitem,
-    (Dtfree_f) free_nitem,
-    NIL(Dtcompar_f),
-    NIL(Dthash_f),
-    NIL(Dtmemory_f),
-    NIL(Dtevent_f)
+    .key = offsetof(namev_t, name),
+    .size = -1,
+    .link = offsetof(namev_t, link),
+    .makef = (Dtmake_f)make_nitem,
+    .freef = (Dtfree_f)free_nitem,
 };
 
 static userdata_t *genUserdata(char* dfltname)
@@ -190,8 +187,7 @@ static void addToMap(Dt_t * map, char *name, char *uniqueName)
     objp->unique_name = strdup(uniqueName);
 }
 
-static char *mapLookup(Dt_t * nm, char *name)
-{
+static char *mapLookup(Dt_t *nm, const char *name) {
     namev_t *objp = dtmatch(nm, name);
     if (objp)
 	return objp->unique_name;
@@ -199,11 +195,10 @@ static char *mapLookup(Dt_t * nm, char *name)
 	return 0;
 }
 
-static int isAnonGraph(char *name)
-{
+static int isAnonGraph(const char *name) {
     if (*name++ != '%')
 	return 0;
-    while (isdigit(*name))
+    while (isdigit((int)*name))
 	name++;			/* skip over digits */
     return (*name == '\0');
 }
@@ -213,7 +208,7 @@ static void push_subg(Agraph_t * g)
     if (GSP == STACK_DEPTH) {
 	fprintf(stderr, "graphml2gv: Too many (> %d) nestings of subgraphs\n",
 		STACK_DEPTH);
-	exit(1);
+	graphviz_exit(1);
     } else if (GSP == 0)
 	root = g;
     G = Gstack[GSP++] = g;
@@ -224,7 +219,7 @@ static Agraph_t *pop_subg(void)
     Agraph_t *g;
     if (GSP == 0) {
 	fprintf(stderr, "graphml2gv: Gstack underflow in graph parser\n");
-	exit(1);
+	graphviz_exit(1);
     }
     g = Gstack[--GSP];
     if (GSP > 0)
@@ -300,7 +295,7 @@ setNodeAttr(Agnode_t * np, char *name, char *value, userdata_t * ud)
  * The names must always begin with "node:".
  */
 static void
-setGlobalNodeAttr(Agraph_t * g, char *name, char *value, userdata_t * ud)
+setGlobalNodeAttr(Agraph_t * g, char *name, char *value)
 {
     if (strncmp(name, NODELBL, NLBLLEN))
 	fprintf(stderr,
@@ -350,7 +345,7 @@ setEdgeAttr(Agedge_t * ep, char *name, char *value, userdata_t * ud)
  * The names always begin with "edge:".
  */
 static void
-setGlobalEdgeAttr(Agraph_t * g, char *name, char *value, userdata_t * ud)
+setGlobalEdgeAttr(Agraph_t * g, char *name, char *value)
 {
     if (strncmp(name, EDGELBL, ELBLLEN))
 	fprintf(stderr,
@@ -449,7 +444,7 @@ startElementHandler(void *userData, const char *name, const char **atts)
 	    push_subg(g);
 	} else {
 	    Agraph_t *subg;
-	    if (isAnonGraph((char *) id)) {
+	    if (isAnonGraph(id)) {
 		static int anon_id = 1;
 		snprintf(buf, sizeof(buf), "%%%d", anon_id++);
 		id = buf;
@@ -486,11 +481,11 @@ startElementHandler(void *userData, const char *name, const char **atts)
 	if (pos > 0)
 	    head = atts[pos];
 
-	tname = mapLookup(ud->nameMap, (char *) tail);
+	tname = mapLookup(ud->nameMap, tail);
 	if (tname)
 	    tail = tname;
 
-	tname = mapLookup(ud->nameMap, (char *) head);
+	tname = mapLookup(ud->nameMap, head);
 	if (tname)
 	    head = tname;
 
@@ -574,10 +569,10 @@ static void endElementHandler(void *userData, const char *name)
 	    setAttr(name, value, ud);
 	    break;
 	case TAG_NODE:
-	    setGlobalNodeAttr(G, name, value, ud);
+	    setGlobalNodeAttr(G, name, value);
 	    break;
 	case TAG_EDGE:
-	    setGlobalEdgeAttr(G, name, value, ud);
+	    setGlobalEdgeAttr(G, name, value);
 	    break;
 	case TAG_GRAPH:
 	    setGraphAttr(G, name, value, ud);
@@ -669,7 +664,7 @@ static FILE *openFile(const char *name)
     if (!fp) {
 	fprintf(stderr, "%s: could not open file %s for writing\n", CmdName, name);
 	perror(name);
-	exit(1);
+	graphviz_exit(1);
     }
     return fp;
 }
@@ -683,7 +678,7 @@ static const char *use = "Usage: %s [-gd?] [-o<file>] [<graphs>]\n\
 static void usage(int v)
 {
     fprintf(stderr, use, CmdName);
-    exit(v);
+    graphviz_exit(v);
 }
 
 static char *cmdName(char *path)
@@ -782,9 +777,9 @@ int main(int argc, char **argv)
 	    fflush(outFile);
 	}
     }
-    exit(rv);
+    graphviz_exit(rv);
 #else
     fputs("cvtgxl: not configured for conversion from GXL to GV\n", stderr);
-    exit(1);
+    graphviz_exit(1);
 #endif
 }
