@@ -21,22 +21,24 @@
 
 #include <assert.h>
 #include <gvc/gvc.h>
+#include <cgraph/exit.h>
 #include <common/render.h>
 #include <neatogen/neatoprocs.h>
 #include <ingraphs/ingraphs.h>
 #include <pack/pack.h>
 #include <stddef.h>
+#include <stdbool.h>
 
-#if defined(_WIN32)
+#ifdef GVDLL
   __declspec(dllimport)
 #endif
 extern gvplugin_library_t gvplugin_neato_layout_LTX_library;
 
 lt_symlist_t lt_preloaded_symbols[] = {
-#if defined(_WIN32)
+#ifdef GVDLL
 	{ "gvplugin_neato_layout_LTX_library", 0 },
 #else
-	{ "gvplugin_neato_layout_LTX_library", (void*)(&gvplugin_neato_layout_LTX_library) },
+	{ "gvplugin_neato_layout_LTX_library", &gvplugin_neato_layout_LTX_library },
 #endif
 	{ 0, 0 }
 };
@@ -71,7 +73,7 @@ static Agdesc_t kind;		/* type of graph */
 static int G_cnt;		/* No. of -G arguments */
 static int G_sz;		/* Storage size for -G arguments */
 static attr_t *G_args;		/* Storage for -G arguments */
-static int doPack;              /* Do packing if true */
+static bool doPack;              /* Do packing if true */
 static char* gname = "root";
 
 #define NEWNODE(n) ((node_t*)ND_alg(n))
@@ -93,7 +95,7 @@ If no files are specified, stdin is used\n";
 static void usage(int v)
 {
     printf("%s",useString);
-    exit(v);
+    graphviz_exit(v);
 }
 
 static FILE *openFile(const char *name)
@@ -103,7 +105,7 @@ static FILE *openFile(const char *name)
     fp = fopen(name, "w");
     if (!fp) {
 	fprintf(stderr, "gvpack: could not open file %s for writing\n", name);
-	exit(1);
+	graphviz_exit(1);
     }
     return (fp);
 }
@@ -172,7 +174,7 @@ static Agsym_t *agedgeattr(Agraph_t *g, char *name, char *value)
  */
 static void init(int argc, char *argv[], pack_info* pinfo)
 {
-    int c, len;
+    int c;
     char buf[BUFSIZ];
     char* bp;
 
@@ -180,14 +182,14 @@ static void init(int argc, char *argv[], pack_info* pinfo)
     pinfo->mode = l_clust;
     pinfo->margin = CL_OFFSET;
     pinfo->doSplines = TRUE; /* Use edges in packing */
-    pinfo->fixed = 0;
+    pinfo->fixed = NULL;
     pinfo->sz = 0;
 
     opterr = 0;
     while ((c = getopt(argc, argv, ":na:gvum:s:o:G:?")) != -1) {
 	switch (c) {
-	case 'a':
-	    len = strlen(optarg) + 2;
+	case 'a': {
+	    size_t len = strlen(optarg) + 2;
 	    if (len > BUFSIZ)
 		bp = N_GNEW(len, char);
 	    else
@@ -197,6 +199,7 @@ static void init(int argc, char *argv[], pack_info* pinfo)
 	    if (bp != buf)
 		free (bp);
 	    break;
+	}
 	case 'n':
 	    parsePackModeInfo ("node", pinfo->mode, pinfo);
 	    break;
@@ -282,8 +285,7 @@ static void init_node_edge(Agraph_t * g)
  * libcommon. If fill is true, we use init_nop (neato -n) to
  * read in attributes relevant to the layout.
  */
-static void init_graph(Agraph_t * g, boolean fill, GVC_t* gvc)
-{
+static void init_graph(Agraph_t *g, bool fill, GVC_t *gvc) {
     int d;
     node_t *n;
     edge_t *e;
@@ -292,12 +294,12 @@ static void init_graph(Agraph_t * g, boolean fill, GVC_t* gvc)
     aginit (g, AGNODE, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);
     aginit (g, AGEDGE, "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);
     GD_gvc(g) = gvc;
-    graph_init(g, FALSE);
+    graph_init(g, false);
     d = late_int(g, agfindgraphattr(g, "dim"), 2, 2);
     if (d != 2) {
 	fprintf(stderr, "Error: graph %s has dim = %d (!= 2)\n", agnameof(g),
 		d);
-	exit(1);
+	graphviz_exit(1);
     }
     Ndim = GD_ndim(g) = 2;
     init_node_edge(g);
@@ -310,7 +312,7 @@ static void init_graph(Agraph_t * g, boolean fill, GVC_t* gvc)
 	    else if (ret > 0)
 		fprintf(stderr, "gvpack does not support backgrounds as found in graph %s\n",
 		    agnameof(g));
-	    exit(1);
+	    graphviz_exit(1);
 	}
 	if (Concentrate) { /* check for edges without pos info */
 	    for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
@@ -408,6 +410,9 @@ static void cloneCluster(Agraph_t * old, Agraph_t * new)
  */
 static void freef(Dt_t * dt, void * obj, Dtdisc_t * disc)
 {
+    (void)dt;
+    (void)disc;
+
     free(obj);
 }
 
@@ -511,28 +516,6 @@ static void cloneGraphAttr(Agraph_t * g, Agraph_t * ng)
     cloneDfltAttrs(g, ng, AGEDGE);
 }
 
-#ifdef UNIMPL
-/* redoBB:
- * If "bb" attribute is valid, translate to reflect graph's
- * new packed position.
- */
-static void redoBB(Agraph_t * g, char *s, Agsym_t * G_bb, point delta)
-{
-    box bb;
-    char buf[100];
-
-    if (sscanf(s, "%d,%d,%d,%d", &bb.LL.x, &bb.LL.y, &bb.UR.x, &bb.UR.y) ==
-	4) {
-	bb.LL.x += delta.x;
-	bb.LL.y += delta.y;
-	bb.UR.x += delta.x;
-	bb.UR.y += delta.y;
-	snprintf(buf, sizeof(buf), "%d,%d,%d,%d", bb.LL.x, bb.LL.y, bb.UR.x, bb.UR.y);
-	agxset(g, G_bb->index, buf);
-    }
-}
-#endif
-
 /* xName:
  * Create a name for an object in the new graph using the
  * dictionary names and the old name. If the old name has not
@@ -543,10 +526,10 @@ static void redoBB(Agraph_t * g, char *s, Agsym_t * G_bb, point delta)
 static char *xName(Dt_t * names, char *oldname)
 {
     static char* name = NULL;
-    static int namelen = 0;
+    static size_t namelen = 0;
     char *namep;
     pair_t *p;
-    int len;
+    size_t len;
 
     p = dtmatch(names, oldname);
     if (p) {
@@ -597,7 +580,7 @@ cloneSubg(Agraph_t * g, Agraph_t * ng, Agsym_t * G_bb, Dt_t * gnames)
     /* clone subgraphs */
     for (subg = agfstsubg (g); subg; subg = agnxtsubg (subg)) {
 	nsubg = agsubg(ng, xName(gnames, agnameof(subg)), 1);
-	agbindrec (nsubg, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);
+	agbindrec (nsubg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
 	cloneSubg(subg, nsubg, G_bb, gnames);
 	/* if subgraphs are clusters, point to the new 
 	 * one so we can find it later.
@@ -622,7 +605,7 @@ cloneSubg(Agraph_t * g, Agraph_t * ng, Agsym_t * G_bb, Dt_t * gnames)
 	    nt = NEWNODE(agtail(e));
 	    nh = NEWNODE(aghead(e));
 	    ne = agedge(ng, nt, nh, NULL, 1);
-	    agbindrec (ne, "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);
+	    agbindrec (ne, "Agedgeinfo_t", sizeof(Agedgeinfo_t), true);
 	    cloneEdge(e, ne);
 	    MARK(e);
 	}
@@ -679,7 +662,7 @@ static Agraph_t *cloneGraph(Agraph_t ** gs, int cnt, GVC_t * gvc)
     Dt_t *nnames;		/* dict of used node names */
     Agsym_t *G_bb;
     Agsym_t *rv;
-    boolean doWarn = TRUE;
+    bool doWarn = true;
 
     if (verbose)
 	fprintf(stderr, "Creating clone graph\n");
@@ -698,7 +681,7 @@ static Agraph_t *cloneGraph(Agraph_t ** gs, int cnt, GVC_t * gvc)
     }
 
     /* do common initialization. This will handle root's label. */
-    init_graph(root, FALSE, gvc);
+    init_graph(root, false, gvc);
     State = GVSPLINES;
 
     gnames = dtopen(&pairdisc, Dtoset);
@@ -717,17 +700,17 @@ static Agraph_t *cloneGraph(Agraph_t ** gs, int cnt, GVC_t * gvc)
 			"Warning: node %s in graph[%d] %s already defined\n",
 			agnameof(n), i, agnameof(g));
 		fprintf(stderr, "Some nodes will be renamed.\n");
-		doWarn = FALSE;
+		doWarn = false;
 	    }
 	    np = agnode(root, xName(nnames, agnameof(n)), 1);
-	    agbindrec (np, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);
+	    agbindrec (np, "Agnodeinfo_t", sizeof(Agnodeinfo_t), true);
 	    ND_alg(n) = np;
 	    cloneNode(n, np);
 	}
 
 	/* wrap the clone of g in a subgraph of root */
 	subg = agsubg(root, xName(gnames, agnameof(g)), 1);
-	agbindrec (subg, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);
+	agbindrec (subg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
 	cloneSubg(g, subg, G_bb, gnames);
     }
     dtclose(gnames);
@@ -799,7 +782,7 @@ static Agraph_t **readGraphs(int *cp, GVC_t* gvc)
 	else if (kind.directed != g->desc.directed) {
 	    fprintf(stderr,
 		    "Error: all graphs must be directed or undirected\n");
-	    exit(1);
+	    graphviz_exit(1);
 	} else if (!agisstrict(g))
 	    kind = g->desc;
 	init_graph(g, doPack, gvc);
@@ -872,18 +855,18 @@ int main(int argc, char *argv[])
     doPack = (pinfo.mode != l_undef);
 
 #if defined(_WIN32)
-    lt_preloaded_symbols[0].address = (void*)(&gvplugin_neato_layout_LTX_library);
+    lt_preloaded_symbols[0].address = &gvplugin_neato_layout_LTX_library;
 #endif
     gvc = gvContextPlugins(lt_preloaded_symbols, DEMAND_LOADING);
     gs = readGraphs(&cnt, gvc);
     if (cnt == 0)
-	exit(0);
+	graphviz_exit(0);
 
     /* pack graphs */
     if (doPack) {
 	if (packGraphs(cnt, gs, 0, &pinfo)) {
 	    fprintf(stderr, "gvpack: packing of graphs failed.\n");
-	    exit(1);
+	    graphviz_exit(1);
 	}
     }
 
@@ -897,5 +880,5 @@ int main(int argc, char *argv[])
 	attach_attrs(g);
     }
     agwrite(g, outfp);
-    exit(0);
+    graphviz_exit(0);
 }
