@@ -8,13 +8,17 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-
+#include    <assert.h>
 #include    "convert.h"
 #include    <cgraph/agxbuf.h>
+#include    <cgraph/exit.h>
+#include    <common/memory.h>
 #ifdef HAVE_EXPAT
 #include    <expat.h>
 #include    <ctype.h>
+#include    <limits.h>
 #include    <stdbool.h>
+#include    <stdlib.h>
 
 #ifndef XML_STATUS_ERROR
 #define XML_STATUS_ERROR 0
@@ -49,22 +53,12 @@ struct slist {
     char buf[1];
 };
 
-static void *gcalloc(size_t nmemb, size_t size)
-{
-    char *rv = calloc(nmemb, size);
-    if (rv == NULL) {
-	fprintf(stderr, "out of memory\n");
-	exit(EXIT_FAILURE);
-    }
-    return rv;
-}
-
 /* Round x up to next multiple of y, which is a power of 2 */
 #define ROUND2(x,y) (((x) + ((y)-1)) & ~((y)-1))
 
 static void pushString(slist ** stk, const char *s)
 {
-    int sz = ROUND2(sizeof(slist) + strlen(s), sizeof(void *));
+    size_t sz = ROUND2(sizeof(slist) + strlen(s), sizeof(void *));
     slist *sp = gcalloc(sz, sizeof(char));
     strcpy(sp->buf, s);
     sp->next = *stk;
@@ -76,7 +70,7 @@ static void popString(slist ** stk)
     slist *sp = *stk;
     if (!sp) {
 	fprintf(stderr, "PANIC: gxl2gv: empty element stack\n");
-	exit(1);
+	graphviz_exit(1);
     }
     *stk = sp->next;
     free(sp);
@@ -86,7 +80,7 @@ static char *topString(slist * stk)
 {
     if (!stk) {
 	fprintf(stderr, "PANIC: gxl2gv: empty element stack\n");
-	exit(1);
+	graphviz_exit(1);
     }
     return stk->buf;
 }
@@ -102,7 +96,7 @@ static void freeString(slist * stk)
     }
 }
 
-typedef struct userdata {
+typedef struct {
     agxbuf xml_attr_name;
     agxbuf xml_attr_value;
     agxbuf composite_buffer;
@@ -132,6 +126,9 @@ typedef struct {
 
 static namev_t *make_nitem(Dt_t * d, namev_t * objp, Dtdisc_t * disc)
 {
+    (void)d;
+    (void)disc;
+
     namev_t *np = malloc(sizeof(namev_t));
     if (np == NULL)
 	return NULL;
@@ -142,20 +139,19 @@ static namev_t *make_nitem(Dt_t * d, namev_t * objp, Dtdisc_t * disc)
 
 static void free_nitem(Dt_t * d, namev_t * np, Dtdisc_t * disc)
 {
+    (void)d;
+    (void)disc;
+
     free(np->unique_name);
     free(np);
 }
 
 static Dtdisc_t nameDisc = {
-    offsetof(namev_t, name),
-    -1,
-    offsetof(namev_t, link),
-    (Dtmake_f) make_nitem,
-    (Dtfree_f) free_nitem,
-    NIL(Dtcompar_f),
-    NIL(Dthash_f),
-    NIL(Dtmemory_f),
-    NIL(Dtevent_f)
+    .key = offsetof(namev_t, name),
+    .size = -1,
+    .link = offsetof(namev_t, link),
+    .makef = (Dtmake_f)make_nitem,
+    .freef = (Dtfree_f)free_nitem,
 };
 
 static userdata_t *genUserdata(void)
@@ -197,8 +193,7 @@ static void addToMap(Dt_t * map, char *name, char *uniqueName)
     objp->unique_name = strdup(uniqueName);
 }
 
-static char *mapLookup(Dt_t * nm, char *name)
-{
+static char *mapLookup(Dt_t *nm, const char *name) {
     namev_t *objp = dtmatch(nm, name);
     if (objp)
 	return objp->unique_name;
@@ -206,11 +201,11 @@ static char *mapLookup(Dt_t * nm, char *name)
 	return 0;
 }
 
-static int isAnonGraph(char *name)
+static int isAnonGraph(const char *name)
 {
     if (*name++ != '%')
 	return 0;
-    while (isdigit(*name))
+    while (isdigit((int)*name))
 	name++;			/* skip over digits */
     return (*name == '\0');
 }
@@ -220,7 +215,7 @@ static void push_subg(Agraph_t * g)
     if (GSP == STACK_DEPTH) {
 	fprintf(stderr, "gxl2gv: Too many (> %d) nestings of subgraphs\n",
 		STACK_DEPTH);
-	exit(1);
+	graphviz_exit(1);
     } else if (GSP == 0)
 	root = g;
     G = Gstack[GSP++] = g;
@@ -231,7 +226,7 @@ static Agraph_t *pop_subg(void)
     Agraph_t *g;
     if (GSP == 0) {
 	fprintf(stderr, "gxl2gv: Gstack underflow in graph parser\n");
-	exit(1);
+	graphviz_exit(1);
     }
     g = Gstack[--GSP];
     if (GSP > 0)
@@ -314,7 +309,7 @@ setNodeAttr(Agnode_t * np, char *name, char *value, userdata_t * ud,
  * The names must always begin with "node:".
  */
 static void
-setGlobalNodeAttr(Agraph_t * g, char *name, char *value, userdata_t * ud)
+setGlobalNodeAttr(Agraph_t * g, char *name, char *value)
 {
     if (strncmp(name, NODELBL, NLBLLEN))
 	fprintf(stderr,
@@ -370,7 +365,7 @@ setEdgeAttr(Agedge_t * ep, char *name, char *value, userdata_t * ud,
  * The names always begin with "edge:".
  */
 static void
-setGlobalEdgeAttr(Agraph_t * g, char *name, char *value, userdata_t * ud)
+setGlobalEdgeAttr(Agraph_t * g, char *name, char *value)
 {
     if (strncmp(name, EDGELBL, ELBLLEN))
 	fprintf(stderr,
@@ -428,7 +423,7 @@ static void
 startElementHandler(void *userData, const char *name, const char **atts)
 {
     int pos;
-    userdata_t *ud = (userdata_t *) userData;
+    userdata_t *ud = userData;
     Agraph_t *g = NULL;
 
     if (strcmp(name, "gxl") == 0) {
@@ -463,7 +458,7 @@ startElementHandler(void *userData, const char *name, const char **atts)
 	    push_subg(g);
 	} else {
 	    Agraph_t *subg;
-	    if (isAnonGraph((char *) id)) {
+	    if (isAnonGraph(id)) {
 		static int anon_id = 1;
 		snprintf(buf, sizeof(buf), "%%%d", anon_id++);
 		id = buf;
@@ -508,11 +503,11 @@ startElementHandler(void *userData, const char *name, const char **atts)
 	if (pos > 0)
 	    head = atts[pos];
 
-	tname = mapLookup(ud->nameMap, (char *) tail);
+	tname = mapLookup(ud->nameMap, tail);
 	if (tname)
 	    tail = tname;
 
-	tname = mapLookup(ud->nameMap, (char *) head);
+	tname = mapLookup(ud->nameMap, head);
 	if (tname)
 	    head = tname;
 
@@ -598,7 +593,7 @@ startElementHandler(void *userData, const char *name, const char **atts)
 
 static void endElementHandler(void *userData, const char *name)
 {
-    userdata_t *ud = (userdata_t *) userData;
+    userdata_t *ud = userData;
 
     if (strcmp(name, "graph") == 0) {
 	pop_subg();
@@ -620,45 +615,45 @@ static void endElementHandler(void *userData, const char *name)
 	ud->closedElementType = TAG_EDGE;
 	ud->edgeinverted = FALSE;
     } else if (strcmp(name, "attr") == 0) {
-	char *name;
+	char *new_name;
 	char *value;
 	char buf[SMALLBUF] = GXL_COMP;
 	char *dynbuf = 0;
 
 	ud->closedElementType = TAG_NONE;
 	if (ud->compositeReadState) {
-	    int len = sizeof(GXL_COMP) + agxblen(&ud->xml_attr_name);
+	    size_t len = sizeof(GXL_COMP) + (size_t)agxblen(&ud->xml_attr_name);
 	    if (len <= SMALLBUF) {
-		name = buf;
+		new_name = buf;
 	    } else {
-		name = dynbuf = gcalloc(len, sizeof(char));
-		strcpy(name, GXL_COMP);
+		new_name = dynbuf = gcalloc(len, sizeof(char));
+		strcpy(new_name, GXL_COMP);
 	    }
-	    strcpy(name + sizeof(GXL_COMP) - 1,
+	    strcpy(new_name + sizeof(GXL_COMP) - 1,
 		   agxbuse(&ud->xml_attr_name));
 	    value = agxbuse(&ud->composite_buffer);
 	    agxbclear(&ud->xml_attr_value);
 	    ud->compositeReadState = FALSE;
 	} else {
-	    name = agxbuse(&ud->xml_attr_name);
+	    new_name = agxbuse(&ud->xml_attr_name);
 	    value = agxbuse(&ud->xml_attr_value);
 	}
 
 	switch (ud->globalAttrType) {
 	case TAG_NONE:
-	    setAttr(name, value, ud, false);
+	    setAttr(new_name, value, ud, false);
 	    break;
 	case TAG_NODE:
-	    setGlobalNodeAttr(G, name, value, ud);
+	    setGlobalNodeAttr(G, new_name, value);
 	    break;
 	case TAG_EDGE:
-	    setGlobalEdgeAttr(G, name, value, ud);
+	    setGlobalEdgeAttr(G, new_name, value);
 	    break;
 	case TAG_GRAPH:
-	    setGraphAttr(G, name, value, ud);
+	    setGraphAttr(G, new_name, value, ud);
 	    break;
 	case TAG_HTML_LIKE_STRING:
-	    setAttr(name, value, ud, true);
+	    setAttr(new_name, value, ud, true);
 	    break;
 	}
 	free(dynbuf);
@@ -680,17 +675,20 @@ static void endElementHandler(void *userData, const char *name)
 
 static void characterDataHandler(void *userData, const char *s, int length)
 {
-    userdata_t *ud = (userdata_t *) userData;
+    userdata_t *ud = userData;
+
+    assert(length >= 0 && "Expat returned negative length data");
+    size_t len = (size_t)length;
 
     if (!ud->listen)
 	return;
 
     if (ud->compositeReadState) {
-	agxbput_n(&ud->composite_buffer, s, length);
+	agxbput_n(&ud->composite_buffer, s, len);
 	return;
     }
 
-    agxbput_n(&ud->xml_attr_value, s, length);
+    agxbput_n(&ud->xml_attr_value, s, len);
 }
 
 Agraph_t *gxl_to_gv(FILE * gxlFile)
@@ -702,7 +700,7 @@ Agraph_t *gxl_to_gv(FILE * gxlFile)
 
     if (udata == NULL) {
 	fprintf(stderr, "out of memory\n");
-	exit(1);
+	graphviz_exit(1);
     }
 
     XML_SetUserData(parser, udata);
@@ -716,12 +714,13 @@ Agraph_t *gxl_to_gv(FILE * gxlFile)
 	if (len == 0)
 	    break;
 	done = len < sizeof(buf);
-	if (XML_Parse(parser, buf, len, done) == XML_STATUS_ERROR) {
+	assert(len <= (size_t)INT_MAX && "too large data for Expat API");
+	if (XML_Parse(parser, buf, (int)len, done) == XML_STATUS_ERROR) {
 	    fprintf(stderr,
 		    "%s at line %lu\n",
 		    XML_ErrorString(XML_GetErrorCode(parser)),
 		    XML_GetCurrentLineNumber(parser));
-	    exit(1);
+	    graphviz_exit(1);
 	}
     } while (!done);
     XML_ParserFree(parser);
