@@ -16,6 +16,7 @@
 #include <unistd.h>
 #endif
 #include "cgraph.h"
+#include "exit.h"
 #include "ingraphs.h"
 
 #include <getopt.h>
@@ -141,7 +142,7 @@ static int visit(Agnode_t * n, Agraph_t * map, Stack * sp, sccstate * st)
 			Agraph_t *G = agraphof(n);;
 			snprintf(name, sizeof(name), "cluster_%d", (st->Comp)++);
 			subg = agsubg(G, name, TRUE);
-			agbindrec(subg, "scc_graph", sizeof(Agraphinfo_t), TRUE);
+			agbindrec(subg, "scc_graph", sizeof(Agraphinfo_t), true);
 			setrep(subg, agnode(map, name, TRUE));
 			do {
 				t = pop(sp);
@@ -252,20 +253,14 @@ static void process(Agraph_t * G)
 
 }
 
-static FILE *openFile(char *name, char *mode)
+static FILE *openFile(const char *name)
 {
 	FILE *fp;
-	char *modestr;
 
-	fp = fopen(name, mode);
+	fp = fopen(name, "w");
 	if (!fp) {
-		if (*mode == 'r')
-			modestr = "reading";
-		else
-			modestr = "writing";
-		fprintf(stderr, "gvpack: could not open file %s for %s\n",
-				name, modestr);
-		exit(1);
+		fprintf(stderr, "gvpack: could not open file %s for writing\n", name);
+		graphviz_exit(1);
 	}
 	return (fp);
 }
@@ -282,7 +277,7 @@ If no files are specified, stdin is used\n";
 static void usage(int v)
 {
 	printf(useString, CmdName);
-	exit(v);
+	graphviz_exit(v);
 }
 
 static void scanArgs(int argc, char **argv)
@@ -300,7 +295,7 @@ static void scanArgs(int argc, char **argv)
 			wantDegenerateComp = 1;
 			break;
 		case 'o':
-			outfp = openFile(optarg, "w");
+			outfp = openFile(optarg);
 			break;
 		case 'v':
 			Verbose = 1;
@@ -331,7 +326,7 @@ static void scanArgs(int argc, char **argv)
 
 static Agraph_t *gread(FILE * fp)
 {
-	return agread(fp, (Agdisc_t *) 0);
+	return agread(fp, NULL);
 }
 
 int main(int argc, char **argv)
