@@ -35,8 +35,10 @@
 #include <common/pointset.h>
 #include <common/intset.h>
 #include <cdt/cdt.h>
+#include <cgraph/itos.h>
 #include <cgraph/strcasecmp.h>
 #include <stddef.h>
+#include <stdbool.h>
 
 #define DEFAULT_BORDER    1
 #define DEFAULT_CELLPADDING  2
@@ -47,7 +49,7 @@ typedef struct {
     char *tooltip;
     char *target;
     char *id;
-    boolean explicit_tooltip;
+    bool explicit_tooltip;
     point LL;
     point UR;
 } htmlmap_data_t;
@@ -389,7 +391,7 @@ initAnchor(GVJ_t * job, htmlenv_t * env, htmldata_t * data, boxf b,
 	agxbinit(&xb, SMALLBUF, buf);
 	if (!env->objid) {
 	    env->objid = strdup(getObjId(job, obj->u.n, &xb));
-	    env->objid_set = 1;
+	    env->objid_set = true;
 	}
 	agxbprint(&xb, "%s_%d", env->objid, anchorId++);
 	id = agxbuse(&xb);
@@ -618,7 +620,7 @@ static void emit_html_img(GVJ_t * job, htmlimg_t * cp, htmlenv_t * env)
 	scale = env->imgscale;
     assert(cp->src);
     assert(cp->src[0]);
-    gvrender_usershape(job, cp->src, A, 4, TRUE, scale, "mc");
+    gvrender_usershape(job, cp->src, A, 4, true, scale, "mc");
 }
 
 static void emit_html_cell(GVJ_t * job, htmlcell_t * cp, htmlenv_t * env)
@@ -765,7 +767,7 @@ void emit_html_label(GVJ_t * job, htmllabel_t * lp, textlabel_t * tp)
     env.finfo.size = tp->fontsize;
     env.imgscale = agget(job->obj->u.n, "imagescale");
     env.objid = job->obj->id;
-    env.objid_set = 0;
+    env.objid_set = false;
     if ((env.imgscale == NULL) || (env.imgscale[0] == '\0'))
 	env.imgscale = "false";
     if (lp->kind == HTML_TBL) {
@@ -937,64 +939,6 @@ boxf *html_port(node_t * n, char *pname, int *sides)
 
 }
 
-/* html_path:
- * Return a box in a table containing the given endpoint.
- * If the top flow is text (no internal structure), return 
- * the box of the flow
- * Else return the box of the subtable containing the point.
- * Because of spacing, the point might not be in any subtable.
- * In that case, return the top flow's box.
- * Note that box[0] must contain the edge point. Additional boxes
- * move out to the boundary.
- *
- * At present, unimplemented, since the label may be inside a
- * non-box node and we need to figure out what this means.
- */
-int html_path(node_t * n, port * p, int side, boxf * rv, int *k)
-{
-#ifdef UNIMPL
-    point p;
-    tbl_t *info;
-    tbl_t *t;
-    boxf b;
-    int i;
-
-    info = (tbl_t *) ND_shape_info(n);
-    assert(info->tbls);
-    info = info->tbls[0];	/* top-level flow */
-    assert(IS_FLOW(info));
-
-    b = info->box;
-    if (info->tbl) {
-	info = info->tbl;
-	if (pt == 1)
-	    p = ED_tail_port(e).p;
-	else
-	    p = ED_head_port(e).p;
-	p = flip_pt(p, GD_rankdir(n->graph));	/* move p to node's coordinate system */
-	for (i = 0; (t = info->tbls[i]) != 0; i++)
-	    if (INSIDE(p, t->box)) {
-		b = t->box;
-		break;
-	    }
-    }
-
-    /* move box into layout coordinate system */
-    if (GD_flip(n->graph))
-	b = flip_trans_box(b, ND_coord_i(n));
-    else
-	b = move_box(b, ND_coord_i(n));
-
-    *k = 1;
-    *rv = b;
-    if (pt == 1)
-	return BOTTOM;
-    else
-	return TOP;
-#endif
-    return 0;
-}
-
 static int size_html_txt(GVC_t *gvc, htmltxt_t * ftxt, htmlenv_t * env)
 {
     double xsize = 0.0;		/* width of text block */
@@ -1151,7 +1095,7 @@ static int size_html_img(htmlimg_t * img, htmlenv_t * env)
 	agerr(AGERR, "No or improper image file=\"%s\"\n", img->src);
     } else {
 	rv = 0;
-	GD_has_images(env->g) = TRUE;
+	GD_has_images(env->g) = true;
     }
 
     B2BF(b, img->box);
@@ -1361,25 +1305,6 @@ static void sizeLinearArray(htmltbl_t * tbl)
     }
 }
 
-static char *nnames[] = {
-    "0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
-    "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20",
-};
-
-/* nToName:
- * Convert int to its decimal string representation.
- */
-static char *nToName(int c)
-{
-    static char name[100];
-
-    if (c < sizeof(nnames) / sizeof(char *))
-	return nnames[c];
-
-    snprintf(name, sizeof(name), "%d", c);
-    return name;
-}
-
 /* closeGraphs:
  * Clean up graphs made for setting column and row widths.
  */
@@ -1408,7 +1333,7 @@ static void checkChain(graph_t * g)
     for (h = ND_next(t); h; h = ND_next(h)) {
 	if (!agfindedge(g, t, h)) {
 	    e = agedge(g, t, h, NULL, 1);
-	    agbindrec(e, "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);
+	    agbindrec(e, "Agedgeinfo_t", sizeof(Agedgeinfo_t), true);
 	    ED_minlen(e) = 0;
 	    elist_append(e, ND_out(t));
 	    elist_append(e, ND_in(h));
@@ -1431,7 +1356,7 @@ checkEdge (graph_t* g, node_t* t, node_t* h, int sz)
 	ED_minlen(e) = MAX(ED_minlen(e), sz);
     else {
 	e = agedge(g, t, h, NULL, 1);
-	agbindrec(e, "Agedgeinfo_t", sizeof(Agedgeinfo_t), TRUE);
+	agbindrec(e, "Agedgeinfo_t", sizeof(Agedgeinfo_t), true);
 	ED_minlen(e) = sz;
 	elist_append(e, ND_out(t));
 	elist_append(e, ND_in(h));
@@ -1458,8 +1383,8 @@ static void makeGraphs(htmltbl_t * tbl, graph_t * rowg, graph_t * colg)
 
     lastn = NULL;
     for (i = 0; i <= tbl->cc; i++) {
-	t = agnode(colg, nToName(i), 1);
-	agbindrec(t, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);
+	t = agnode(colg, itos(i).str, 1);
+	agbindrec(t, "Agnodeinfo_t", sizeof(Agnodeinfo_t), true);
 	alloc_elist(tbl->rc, ND_in(t));
 	alloc_elist(tbl->rc, ND_out(t));
 	if (lastn) {
@@ -1471,8 +1396,8 @@ static void makeGraphs(htmltbl_t * tbl, graph_t * rowg, graph_t * colg)
     }
     lastn = NULL;
     for (i = 0; i <= tbl->rc; i++) {
-	t = agnode(rowg, nToName(i), 1);
-	agbindrec(t, "Agnodeinfo_t", sizeof(Agnodeinfo_t), TRUE);
+	t = agnode(rowg, itos(i).str, 1);
+	agbindrec(t, "Agnodeinfo_t", sizeof(Agnodeinfo_t), true);
 	alloc_elist(tbl->cc, ND_in(t));
 	alloc_elist(tbl->cc, ND_out(t));
 	if (lastn) {
@@ -1485,12 +1410,12 @@ static void makeGraphs(htmltbl_t * tbl, graph_t * rowg, graph_t * colg)
 
     for (cells = tbl->u.n.cells; *cells; cells++) {
 	cp = *cells;
-	t = agfindnode(colg, nToName(cp->col));
-	h = agfindnode(colg, nToName(cp->col + cp->cspan));
+	t = agfindnode(colg, itos(cp->col).str);
+	h = agfindnode(colg, itos(cp->col + cp->cspan).str);
 	checkEdge (colg, t, h, cp->data.box.UR.x);
 
-	t = agfindnode(rowg, nToName(cp->row));
-	h = agfindnode(rowg, nToName(cp->row + cp->rspan));
+	t = agfindnode(rowg, itos(cp->row).str);
+	h = agfindnode(rowg, itos(cp->row + cp->rspan).str);
 	checkEdge (rowg, t, h, cp->data.box.UR.y);
     }
 
@@ -1554,8 +1479,8 @@ static void sizeArray(htmltbl_t * tbl)
     rowg = agopen("rowg", dir, NULL);
     colg = agopen("colg", dir, NULL);
     /* Only need GD_nlist */
-    agbindrec(rowg, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);	// graph custom data
-    agbindrec(colg, "Agraphinfo_t", sizeof(Agraphinfo_t), TRUE);	// graph custom data
+    agbindrec(rowg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);	// graph custom data
+    agbindrec(colg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);	// graph custom data
     makeGraphs(tbl, rowg, colg);
     rank(rowg, 2, INT_MAX);
     rank(colg, 2, INT_MAX);
@@ -2082,7 +2007,7 @@ int make_html_label(void *obj, textlabel_t * lp)
 	agxbuf xb;
 	unsigned char buf[SMALLBUF];
 	agxbinit(&xb, SMALLBUF, buf);
-	lp->html = FALSE;
+	lp->html = false;
 	lp->text = strdup(nameOf(obj, &xb));
 	switch (lp->charset) {
 	case CHAR_LATIN1:
