@@ -16,6 +16,7 @@
 
 #include <ctype.h>
 #include <stdarg.h>
+#include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 #include <inttypes.h>
@@ -35,7 +36,7 @@
 #ifndef OS_CODE
 #  define OS_CODE  0x03  /* assume Unix */
 #endif
-static char z_file_header[] =
+static const unsigned char z_file_header[] =
    {0x1f, 0x8b, /*magic*/ Z_DEFLATED, 0 /*flags*/, 0,0,0,0 /*time*/, 0 /*xflags*/, OS_CODE};
 
 static z_stream z_strm;
@@ -45,6 +46,7 @@ static uint64_t crc;
 #endif /* HAVE_LIBZ */
 
 #include <assert.h>
+#include <cgraph/exit.h>
 #include <common/const.h>
 #include <common/memory.h>
 #include <gvc/gvplugin_device.h>
@@ -57,8 +59,7 @@ static uint64_t crc;
 
 static const int PAGE_ALIGN = 4095;		/* align to a 4K boundary (less one), typical for Linux, Mac OS X and Windows memory allocation */
 
-static size_t gvwrite_no_z (GVJ_t * job, const char *s, size_t len)
-{
+static size_t gvwrite_no_z(GVJ_t * job, const void *s, size_t len) {
     if (job->gvc->write_fn)   /* externally provided write discipline */
 	return (job->gvc->write_fn)(job, s, len);
     if (job->output_data) {
@@ -68,7 +69,7 @@ static size_t gvwrite_no_z (GVJ_t * job, const char *s, size_t len)
 	    job->output_data = realloc(job->output_data, job->output_data_allocated);
 	    if (!job->output_data) {
                 (job->common->errorfn) ("memory allocation failure\n");
-		exit(1);
+		graphviz_exit(1);
 	    }
 	}
 	memcpy(job->output_data + job->output_data_position, s, len);
@@ -211,7 +212,7 @@ size_t gvwrite (GVJ_t * job, const char *s, size_t len)
 	    df = realloc(df, dfallocated);
 	    if (! df) {
                 (job->common->errorfn) ("memory allocation failure\n");
-		exit(1);
+		graphviz_exit(1);
 	    }
 	}
 
@@ -222,32 +223,32 @@ size_t gvwrite (GVJ_t * job, const char *s, size_t len)
 	while (z->avail_in) {
 	    z->next_out = df;
 	    z->avail_out = dfallocated;
-	    ret=deflate (z, Z_NO_FLUSH);
-	    if (ret != Z_OK) {
-                (job->common->errorfn) ("deflation problem %d\n", ret);
-	        exit(1);
+	    int r = deflate(z, Z_NO_FLUSH);
+	    if (r != Z_OK) {
+                (job->common->errorfn) ("deflation problem %d\n", r);
+	        graphviz_exit(1);
 	    }
 
 	    if ((olen = z->next_out - df)) {
-		ret = gvwrite_no_z (job, (char*)df, olen);
+		ret = gvwrite_no_z(job, df, olen);
 	        if (ret != olen) {
                     (job->common->errorfn) ("gvwrite_no_z problem %d\n", ret);
-	            exit(1);
+	            graphviz_exit(1);
 	        }
 	    }
 	}
 
 #else
-        NOTUSED(olen);
+        (void)olen;
 	(job->common->errorfn) ("No libz support.\n");
-	exit(1);
+	graphviz_exit(1);
 #endif
     }
     else { /* uncompressed write */
 	ret = gvwrite_no_z (job, s, len);
 	if (ret != len) {
 	    (job->common->errorfn) ("gvwrite_no_z problem %d\n", len);
-	    exit(1);
+	    graphviz_exit(1);
 	}
     }
     return len;
@@ -278,6 +279,18 @@ int gvputs_xml(GVJ_t *job, const char *s) {
   return xml_escape(s, flags, (int (*)(void *, const char *))gvputs, job);
 }
 
+void gvputs_nonascii(GVJ_t *job, const char *s) {
+  for (; *s != '\0'; ++s) {
+    if (*s == '\\') {
+      gvputs(job, "\\\\");
+    } else if (isascii((int)*s)) {
+      gvputc(job, *s);
+    } else {
+      gvprintf(job, "%03o", (unsigned)*s);
+    }
+  }
+}
+
 int gvputc(GVJ_t * job, int c)
 {
     const char cc = c;
@@ -324,7 +337,7 @@ void gvdevice_format(GVJ_t * job)
 void gvdevice_finalize(GVJ_t * job)
 {
     gvdevice_engine_t *gvde = job->device.engine;
-    boolean finalized_p = FALSE;
+    bool finalized_p = false;
 
     if (job->flags & GVDEVICE_COMPRESSED_FORMAT) {
 #ifdef HAVE_LIBZ
@@ -338,20 +351,20 @@ void gvdevice_finalize(GVJ_t * job)
 	z->next_out = df;
 	z->avail_out = dfallocated;
 	while ((ret = deflate (z, Z_FINISH)) == Z_OK && (cnt++ <= 100)) {
-	    gvwrite_no_z(job, (char*)df, z->next_out - df);
+	    gvwrite_no_z(job, df, z->next_out - df);
 	    z->next_out = df;
 	    z->avail_out = dfallocated;
 	}
 	if (ret != Z_STREAM_END) {
             (job->common->errorfn) ("deflation finish problem %d cnt=%d\n", ret, cnt);
-	    exit(1);
+	    graphviz_exit(1);
 	}
-	gvwrite_no_z(job, (char*)df, z->next_out - df);
+	gvwrite_no_z(job, df, z->next_out - df);
 
 	ret = deflateEnd(z);
 	if (ret != Z_OK) {
 	    (job->common->errorfn) ("deflation end problem %d\n", ret);
-	    exit(1);
+	    graphviz_exit(1);
 	}
 	out[0] = (unsigned char)crc;
 	out[1] = (unsigned char)(crc >> 8);
@@ -361,17 +374,17 @@ void gvdevice_finalize(GVJ_t * job)
 	out[5] = (unsigned char)(z->total_in >> 8);
 	out[6] = (unsigned char)(z->total_in >> 16);
 	out[7] = (unsigned char)(z->total_in >> 24);
-	gvwrite_no_z(job, (char*)out, sizeof(out));
+	gvwrite_no_z(job, out, sizeof(out));
 #else
 	(job->common->errorfn) ("No libz support\n");
-	exit(1);
+	graphviz_exit(1);
 #endif
     }
 
     if (gvde) {
 	if (gvde->finalize) {
 	    gvde->finalize(job);
-	    finalized_p = TRUE;
+	    finalized_p = true;
 	}
     }
 
@@ -405,12 +418,12 @@ void gvprintf(GVJ_t * job, const char *format, ...)
     /* C99 vsnprintf returns the length that would be required
      * to write the string without truncation. 
      */
-	bp = gmalloc(len + 1);
+	bp = gmalloc((size_t)len + 1);
 	len = vsprintf(bp, format, argp);
     }
     va_end(argp);
 
-    gvwrite(job, bp, len);
+    gvwrite(job, bp, (size_t)len);
     if (bp != buf)
 	free (bp);
 }
@@ -436,7 +449,7 @@ static char * gvprintnum (size_t *len, double number)
     static char tmpbuf[sizeof(maxnegnumstr)];   /* buffer big enough for worst case */
     char *result = tmpbuf+sizeof(maxnegnumstr); /* init result to end of tmpbuf */
     long int N;
-    boolean showzeros, negative;
+    bool showzeros, negative;
     int digit, i;
 
     /*
@@ -464,12 +477,12 @@ static char * gvprintnum (size_t *len, double number)
 	*len = 1;
 	return "0";
     }
-    if ((negative = (N < 0)))		/* avoid "-0" by testing rounded int */
+    if ((negative = N < 0))		/* avoid "-0" by testing rounded int */
         N = -N;				/* make number +ve */
 #ifdef TERMINATED_NUMBER_STRING
     *--result = '\0';			/* terminate the result string */
 #endif
-    showzeros = FALSE;			/* don't print trailing zeros */
+    showzeros = false;			/* don't print trailing zeros */
     for (i = DECPLACES; N || i > 0; i--) {  /* non zero remainder,
 						or still in fractional part */
         digit = (int)(N % 10L);			/* next least-significant digit */
@@ -477,12 +490,12 @@ static char * gvprintnum (size_t *len, double number)
         if (digit || showzeros) {	/* if digit is non-zero,
 						or if we are printing zeros */
             *--result = (char)digit | '0';	/* convert digit to ascii */
-            showzeros = TRUE;		/* from now on we must print zeros */
+            showzeros = true;		/* from now on we must print zeros */
         }
         if (i == 1) {			/* if completed fractional part */
             if (showzeros)		/* if there was a non-zero fraction */
                 *--result = '.';	/* print decimal point */
-            showzeros = TRUE;		/* print all digits in int part */
+            showzeros = true;		/* print all digits in int part */
         }
     }
     if (negative)			/* print "-" if needed */
@@ -518,7 +531,7 @@ int main (int argc, char *argv[])
         fprintf (stdout, "%g = %s %d\n", test[i], buf, len);
     }
 
-    return 0;
+    graphviz_exit(0);
 }
 #endif
 
@@ -534,7 +547,7 @@ static void gv_trim_zeros(char* buf)
   }
 
   // check this really is the result of %.02f printing
-  assert(isdigit(dotp[1]) && isdigit(dotp[2]) && dotp[3] == '\0');
+  assert(isdigit((int)dotp[1]) && isdigit((int)dotp[2]) && dotp[3] == '\0');
 
   if (dotp[2] == '0') {
     if (dotp[1] == '0') {
@@ -578,7 +591,7 @@ void gvprintpointflist(GVJ_t * job, pointf *p, int n)
 {
     int i = 0;
 
-    while (TRUE) {
+    while (true) {
 	gvprintpointf(job, p[i]);
         if (++i >= n) break;
         gvwrite(job, " ", 1);
