@@ -8,22 +8,23 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#include <float.h>
 #include <neatogen/multispline.h>
 #include <neatogen/delaunay.h>
 #include <neatogen/neatoprocs.h>
 #include <math.h>
+#include <stdbool.h>
 
-
-static boolean spline_merge(node_t * n)
+static bool spline_merge(node_t * n)
 {
     (void)n;
-    return FALSE;
+    return false;
 }
 
-static boolean swap_ends_p(edge_t * e)
+static bool swap_ends_p(edge_t * e)
 {
     (void)e;
-    return FALSE;
+    return false;
 }
 
 static splineInfo sinfo = {.swapEnds = swap_ends_p,
@@ -56,8 +57,8 @@ typedef struct {
 
 static int cmpItem(Dt_t * d, int p1[], int p2[], Dtdisc_t * disc)
 {
-    NOTUSED(d);
-    NOTUSED(disc);
+    (void)d;
+    (void)disc;
 
     if (p1[0] < p2[0])
 	return -1;
@@ -77,8 +78,8 @@ static void *newItem(Dt_t * d, item * objp, Dtdisc_t * disc)
 {
     item *newp = NEW(item);
 
-    NOTUSED(d);
-    NOTUSED(disc);
+    (void)d;
+    (void)disc;
     newp->a[0] = objp->a[0];
     newp->a[1] = objp->a[1];
     newp->t = objp->t;
@@ -88,21 +89,18 @@ static void *newItem(Dt_t * d, item * objp, Dtdisc_t * disc)
 
 static void freeItem(Dt_t * d, item * obj, Dtdisc_t * disc)
 {
-    NOTUSED(d);
-    NOTUSED(disc);
+    (void)d;
+    (void)disc;
     free(obj);
 }
 
 static Dtdisc_t itemdisc = {
-    offsetof(item, a),
-    2 * sizeof(int),
-    offsetof(item, link),
-    (Dtmake_f) newItem,
-    (Dtfree_f) freeItem,
-    (Dtcompar_f) cmpItem,
-    NIL(Dthash_f),
-    NIL(Dtmemory_f),
-    NIL(Dtevent_f)
+    .key = offsetof(item, a),
+    .size = 2 * sizeof(int),
+    .link = offsetof(item, link),
+    .makef = (Dtmake_f)newItem,
+    .freef = (Dtfree_f)freeItem,
+    .comparf = (Dtcompar_f)cmpItem,
 };
 
 static void addMap(Dt_t * map, int a, int b, int t)
@@ -169,8 +167,8 @@ typedef struct {
 
 static int cmpIpair(Dt_t * d, int *p1, int *p2, Dtdisc_t * disc)
 {
-    NOTUSED(d);
-    NOTUSED(disc);
+    (void)d;
+    (void)disc;
 
     return (*p1 - *p2);
 }
@@ -179,8 +177,8 @@ static void *newIpair(Dt_t * d, Ipair * objp, Dtdisc_t * disc)
 {
     Ipair *newp = NEW(Ipair);
 
-    NOTUSED(d);
-    NOTUSED(disc);
+    (void)d;
+    (void)disc;
     newp->i = objp->i;
     newp->j = objp->j;
 
@@ -189,21 +187,18 @@ static void *newIpair(Dt_t * d, Ipair * objp, Dtdisc_t * disc)
 
 static void freeIpair(Dt_t * d, Ipair * obj, Dtdisc_t * disc)
 {
-    NOTUSED(d);
-    NOTUSED(disc);
+    (void)d;
+    (void)disc;
     free(obj);
 }
 
 static Dtdisc_t ipairdisc = {
-    offsetof(Ipair, i),
-    sizeof(int),
-    offsetof(Ipair, link),
-    (Dtmake_f) newIpair,
-    (Dtfree_f) freeIpair,
-    (Dtcompar_f) cmpIpair,
-    NIL(Dthash_f),
-    NIL(Dtmemory_f),
-    NIL(Dtevent_f)
+    .key = offsetof(Ipair, i),
+    .size = sizeof(int),
+    .link = offsetof(Ipair, link),
+    .makef = (Dtmake_f)newIpair,
+    .freef = (Dtfree_f)freeIpair,
+    .comparf = (Dtcompar_f)cmpIpair,
 };
 
 static void vmapAdd(Dt_t * map, int i, int j)
@@ -284,10 +279,6 @@ raySegIntersect(pointf v, pointf w, pointf a, pointf b, pointf * p)
 	return 0;
 }
 
-#ifdef DEVDBG
-#include <psdbg.c>
-#endif
-
 /* triPoint:
  * Given the triangle vertex v, and point w so that v->w points
  * into the polygon, return where the ray v->w intersects the
@@ -305,16 +296,6 @@ triPoint(tripoly_t * trip, int vx, pointf v, pointf w, pointf * ip)
 	    (v, w, trip->poly.ps[tp->v.i], trip->poly.ps[tp->v.j], ip))
 	    return 0;
     }
-#ifdef DEVDBG
-    psInit();
-    psComment ("Failure in triPoint");
-    psColor("0 0 1");
-    psSeg (v, w);
-    for (tp = trip->triMap[vx]; tp; tp = tp->nxttri) {
-	psTri(v, trip->poly.ps[tp->v.i], trip->poly.ps[tp->v.j]);
-    }
-    psOut(stderr);
-#endif
     return 1;
 }
 
@@ -651,52 +632,6 @@ void freeRouter(router_t * rtr)
     free(rtr);
 }
 
-#ifdef DEVDBG
-static void
-prTriPoly (tripoly_t *poly, int si, int ei)
-{
-    FILE* fp = fopen ("dumppoly","w");
-    
-    psInit();
-    psPoly (&(poly->poly));
-    psPoint (poly->poly.ps[si]);
-    psPoint (poly->poly.ps[ei]);
-    psOut(fp);
-    fclose(fp);
-}
-
-static void
-prTriGraph (router_t* rtr, int n)
-{
-    FILE* fp = fopen ("dump","w");
-    int i;
-    pointf* pts = rtr->ps;
-    tnode* nodes = rtr->tg->nodes;
-    char buf[BUFSIZ];
-    
-    psInit();
-    for (i=0;i < rtr->tn; i++) {
-        pointf a = pts[rtr->tris[3*i]];
-        pointf b = pts[rtr->tris[3*i+1]];
-        pointf c = pts[rtr->tris[3*i+2]];
-	psTri (a, b,c);
-	snprintf(buf, sizeof(buf), "%d", i);
-        psTxt (buf, nodes[i].ctr);
-    }
-    for (i=rtr->tn;i < n; i++) {
-	snprintf(buf, sizeof(buf), "%d", i);
-        psTxt (buf, nodes[i].ctr);
-    }
-    psColor ("1 0 0");
-    for (i=0;i < rtr->tg->nedges; i++) {
-        tedge* e = rtr->tg->edges+i;
-        psSeg (nodes[e->t].ctr, nodes[e->h].ctr);
-    }
-    psOut(fp);
-    fclose(fp);
-}
-#endif
-
 router_t *mkRouter(Ppoly_t** obsp, int npoly)
 {
     router_t *rtr = NEW(router_t);
@@ -779,9 +714,7 @@ router_t *mkRouter(Ppoly_t** obsp, int npoly)
  * Finish edge generation, clipping to nodes and adding arrowhead
  * if necessary, and adding edge labels
  */
-static void
-finishEdge (graph_t* g, edge_t* e, Ppoly_t spl, int flip, pointf p, pointf q)
-{
+static void finishEdge(edge_t* e, Ppoly_t spl, int flip, pointf p, pointf q) {
     int j;
     pointf *spline = N_GNEW(spl.pn, pointf);
     pointf p1, q1;
@@ -805,7 +738,7 @@ finishEdge (graph_t* g, edge_t* e, Ppoly_t spl, int flip, pointf p, pointf q)
     clip_and_install(e, aghead(e), spline, spl.pn, &sinfo);
     free(spline);
 
-    addEdgeLabels(g, e, p1, q1);
+    addEdgeLabels(e, p1, q1);
 }
 
 #define EQPT(p,q) (((p).x==(q).x)&&((p).y==(q).y))
@@ -860,8 +793,7 @@ tweakPath (Ppoly_t poly, int s, int t, Ppolyline_t pl)
  * Edges go from s to t.
  * Return 0 on success.
  */
-static int 
-genroute(graph_t* g, tripoly_t * trip, int s, int t, edge_t * e, int doPolyline)
+static int genroute(tripoly_t * trip, int s, int t, edge_t * e, int doPolyline)
 {
     pointf eps[2];
     Pvector_t evs[2];
@@ -906,7 +838,7 @@ genroute(graph_t* g, tripoly_t * trip, int s, int t, edge_t * e, int doPolyline)
 	    rv = 1;
 	    goto finish;
 	}
-	finishEdge (g, e, spl, aghead(e) != head, eps[0], eps[1]);
+	finishEdge(e, spl, aghead(e) != head, eps[0], eps[1]);
 	free(medges);
 
 	return 0;
@@ -959,7 +891,7 @@ genroute(graph_t* g, tripoly_t * trip, int s, int t, edge_t * e, int doPolyline)
 		goto finish;
 	    }
 	}
-	finishEdge (g, e, spl, aghead(e) != head, eps[0], eps[1]);
+	finishEdge(e, spl, aghead(e) != head, eps[0], eps[1]);
 
 	e = ED_to_virt(e);
     }
@@ -1286,7 +1218,7 @@ typedef struct {
 
 #define N_DAD(n) dad[n]
 #define E_WT(e) (e->dist)
-#define UNSEEN -MAXFLOAT
+#define UNSEEN (-FLT_MAX)
 
 /* triPath:
  * Find the shortest path with lengths in g from
@@ -1344,8 +1276,7 @@ triPath(tgraph * g, int n, int v0, int v1, PQ * pq)
  * so avoid in neato spline code.
  * Return 0 on success.
  */
-int makeMultiSpline(graph_t* g,  edge_t* e, router_t * rtr, int doPolyline)
-{
+int makeMultiSpline(edge_t* e, router_t * rtr, int doPolyline) {
     Ppolyline_t line = ED_path(e);
     node_t *t = agtail(e);
     node_t *h = aghead(e);
@@ -1387,7 +1318,7 @@ int makeMultiSpline(graph_t* g,  edge_t* e, router_t * rtr, int doPolyline)
 	free(sp);
 
 	/* Generate multiple splines using polygon */
-	ret = genroute(g, poly, 0, idx, e, doPolyline);
+	ret = genroute(poly, 0, idx, e, doPolyline);
 	freeTripoly (poly);
     }
     else ret = -1;
