@@ -1,6 +1,3 @@
-/* $Id$Revision: */
-/* vim:set shiftwidth=4 ts=8: */
-
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property
  * All rights reserved. This program and the accompanying materials
@@ -8,7 +5,7 @@
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 
@@ -16,15 +13,14 @@
 
 #include <string.h>
 #include <assert.h>
+#include <stdbool.h>
 #include <stdio.h>
-#ifndef __USE_ISOC99
-#define __USE_ISOC99
-#endif
 #include <math.h>
-#include <geom.h>
-#include <logic.h>
-#include <memory.h>
-#include <trap.h>
+#include <common/geom.h>
+#include <common/logic.h>
+#include <common/memory.h>
+#include <common/types.h>
+#include <ortho/trap.h>
 
 /* Node types */
 
@@ -52,7 +48,6 @@ typedef struct {
   int left, right;      /* children */
 } qnode_t;
 
-/* static int chain_idx, op_idx, mon_idx; */
 static int q_idx;
 static int tr_idx;
 static int QSIZE;
@@ -87,7 +82,7 @@ static int newtrap(trap_t* tr)
 }
 
 /* Return the maximum of the two points into the yval structure */
-static int _max (pointf *yval, pointf *v0, pointf *v1)
+static void _max (pointf *yval, pointf *v0, pointf *v1)
 {
   if (v0->y > v1->y + C_EPS)
     *yval = *v0;
@@ -100,12 +95,10 @@ static int _max (pointf *yval, pointf *v0, pointf *v1)
     }
   else
     *yval = *v1;
-
-  return 0;
 }
 
 /* Return the minimum of the two points into the yval structure */
-static int _min (pointf *yval, pointf *v0, pointf *v1)
+static void _min (pointf *yval, pointf *v0, pointf *v1)
 {
   if (v0->y < v1->y - C_EPS)
     *yval = *v0;
@@ -118,28 +111,21 @@ static int _min (pointf *yval, pointf *v0, pointf *v1)
     }
   else
     *yval = *v1;
-
-  return 0;
 }
 
-static int _greater_than_equal_to (pointf *v0, pointf *v1)
+static bool _greater_than_equal_to (pointf *v0, pointf *v1)
 {
   if (v0->y > v1->y + C_EPS)
     return TRUE;
   else if (v0->y < v1->y - C_EPS)
     return FALSE;
   else
-    return (v0->x >= v1->x);
+    return v0->x >= v1->x;
 }
 
-static int _less_than (pointf *v0, pointf *v1)
+static bool _less_than (pointf *v0, pointf *v1)
 {
-  if (v0->y < v1->y - C_EPS)
-    return TRUE;
-  else if (v0->y > v1->y + C_EPS)
-    return FALSE;
-  else
-    return (v0->x < v1->x);
+  return !_greater_than_equal_to(v0, v1);
 }
 
 /* Initilialise the query structure (Q) and the trapezoid table (T)
@@ -223,7 +209,7 @@ init_query_structure(int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
   qs[i6].trnum = t1;
   qs[i7].trnum = t2;
 
-  s->is_inserted = TRUE;
+  s->is_inserted = true;
   return root;
 }
 
@@ -231,7 +217,7 @@ init_query_structure(int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
  * segnum. Takes care of the degenerate cases when both the vertices
  * have the same y--cood, etc.
  */
-static int
+static bool
 is_left_of (int segnum, segment_t* seg, pointf *v)
 {
   segment_t *s = &seg[segnum];
@@ -254,7 +240,7 @@ is_left_of (int segnum, segment_t* seg, pointf *v)
 	    area = -1.0;
 	}
       else
-	area = CROSS(s->v0, s->v1, (*v));
+	area = CROSS(s->v0, s->v1, *v);
     }
   else				/* v0 > v1 */
     {
@@ -276,16 +262,13 @@ is_left_of (int segnum, segment_t* seg, pointf *v)
 	area = CROSS(s->v1, s->v0, (*v));
     }
 
-  if (area > 0.0)
-    return TRUE;
-  else
-    return FALSE;
+  return area > 0.0;
 }
 
 /* Returns true if the corresponding endpoint of the given segment is */
 /* already inserted into the segment tree. Use the simple test of */
 /* whether the segment which shares this endpoint is already inserted */
-static int inserted (int segnum, segment_t* seg, int whichpt)
+static bool inserted (int segnum, segment_t* seg, int whichpt)
 {
   if (whichpt == FIRSTPT)
     return seg[seg[segnum].prev].is_inserted;
@@ -358,24 +341,25 @@ static void
 merge_trapezoids (int segnum, int tfirst, int tlast, int side, trap_t* tr,
     qnode_t* qs)
 {
-  int t, tnext, cond;
-  int ptnext;
+  int t;
 
   /* First merge polys on the LHS */
   t = tfirst;
-  while ((t > 0) && _greater_than_equal_to(&tr[t].lo, &tr[tlast].lo))
+  while (t > 0 && _greater_than_equal_to(&tr[t].lo, &tr[tlast].lo))
     {
+      int tnext, ptnext;
+      bool cond;
       if (side == S_LEFT)
-	cond = ((((tnext = tr[t].d0) > 0) && (tr[tnext].rseg == segnum)) ||
-		(((tnext = tr[t].d1) > 0) && (tr[tnext].rseg == segnum)));
+	cond = ((tnext = tr[t].d0) > 0 && tr[tnext].rseg == segnum) ||
+		((tnext = tr[t].d1) > 0 && tr[tnext].rseg == segnum);
       else
-	cond = ((((tnext = tr[t].d0) > 0) && (tr[tnext].lseg == segnum)) ||
-		(((tnext = tr[t].d1) > 0) && (tr[tnext].lseg == segnum)));
+	cond = ((tnext = tr[t].d0) > 0 && tr[tnext].lseg == segnum) ||
+		((tnext = tr[t].d1) > 0 && tr[tnext].lseg == segnum);
 
       if (cond)
 	{
-	  if ((tr[t].lseg == tr[tnext].lseg) &&
-	      (tr[t].rseg == tr[tnext].rseg)) /* good neighbours */
+	  if (tr[t].lseg == tr[tnext].lseg &&
+	      tr[t].rseg == tr[tnext].rseg) /* good neighbours */
 	    {			              /* merge them */
 	      /* Use the upper node as the new node i.e. t */
 
@@ -430,7 +414,7 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
   int tfirstr = 0, tlastr = 0, tfirstl = 0, tlastl = 0;
   int i1, i2, t, tn;
   pointf tpt;
-  int tritop = 0, tribot = 0, is_swapped;
+  int tribot = 0, is_swapped;
   int tmptriseg;
 
   s = seg[segnum];
@@ -447,8 +431,8 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
     }
   else is_swapped = FALSE;
 
-  if ((is_swapped) ? !inserted(segnum, seg, LASTPT) :
-       !inserted(segnum, seg, FIRSTPT))     /* insert v0 in the tree */
+  if (!inserted(segnum, seg, is_swapped ? LASTPT : FIRSTPT))
+    /* insert v0 in the tree */
     {
       int tmp_d;
 
@@ -463,14 +447,14 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
       tr[tl].u0 = tu;
       tr[tl].u1 = 0;
 
-      if (((tmp_d = tr[tl].d0) > 0) && (tr[tmp_d].u0 == tu))
+      if ((tmp_d = tr[tl].d0) > 0 && tr[tmp_d].u0 == tu)
 	tr[tmp_d].u0 = tl;
-      if (((tmp_d = tr[tl].d0) > 0) && (tr[tmp_d].u1 == tu))
+      if ((tmp_d = tr[tl].d0) > 0 && tr[tmp_d].u1 == tu)
 	tr[tmp_d].u1 = tl;
 
-      if (((tmp_d = tr[tl].d1) > 0) && (tr[tmp_d].u0 == tu))
+      if ((tmp_d = tr[tl].d1) > 0 && tr[tmp_d].u0 == tu)
 	tr[tmp_d].u0 = tl;
-      if (((tmp_d = tr[tl].d1) > 0) && (tr[tmp_d].u1 == tu))
+      if ((tmp_d = tr[tl].d1) > 0 && tr[tmp_d].u1 == tu)
 	tr[tmp_d].u1 = tl;
 
       /* Now update the query structure and obtain the sinks for the */
@@ -501,12 +485,11 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
   else				/* v0 already present */
     {       /* Get the topmost intersecting trapezoid */
       tfirst = locate_endpoint(&s.v0, &s.v1, s.root0, seg, qs);
-      tritop = 1;
     }
 
 
-  if ((is_swapped) ? !inserted(segnum, seg, FIRSTPT) :
-       !inserted(segnum, seg, LASTPT))     /* insert v1 in the tree */
+  if (!inserted(segnum, seg, is_swapped ? FIRSTPT : LASTPT))
+    /* insert v1 in the tree */
     {
       int tmp_d;
 
@@ -522,14 +505,14 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
       tr[tl].u0 = tu;
       tr[tl].u1 = 0;
 
-      if (((tmp_d = tr[tl].d0) > 0) && (tr[tmp_d].u0 == tu))
+      if ((tmp_d = tr[tl].d0) > 0 && tr[tmp_d].u0 == tu)
 	tr[tmp_d].u0 = tl;
-      if (((tmp_d = tr[tl].d0) > 0) && (tr[tmp_d].u1 == tu))
+      if ((tmp_d = tr[tl].d0) > 0 && tr[tmp_d].u1 == tu)
 	tr[tmp_d].u1 = tl;
 
-      if (((tmp_d = tr[tl].d1) > 0) && (tr[tmp_d].u0 == tu))
+      if ((tmp_d = tr[tl].d1) > 0 && tr[tmp_d].u0 == tu)
 	tr[tmp_d].u0 = tl;
-      if (((tmp_d = tr[tl].d1) > 0) && (tr[tmp_d].u1 == tu))
+      if ((tmp_d = tr[tl].d1) > 0 && tr[tmp_d].u1 == tu)
 	tr[tmp_d].u1 = tl;
 
       /* Now update the query structure and obtain the sinks for the */
@@ -569,8 +552,7 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
 
   t = tfirst;			/* topmost trapezoid */
 
-  while ((t > 0) &&
-	 _greater_than_equal_to(&tr[t].lo, &tr[tlast].lo))
+  while (t > 0 && _greater_than_equal_to(&tr[t].lo, &tr[tlast].lo))
 				/* traverse from top to bot */
     {
       int t_sav, tn_sav;
@@ -605,7 +587,7 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
 
       /* error */
 
-      if ((tr[t].d0 <= 0) && (tr[t].d1 <= 0)) /* case cannot arise */
+      if (tr[t].d0 <= 0 && tr[t].d1 <= 0) /* case cannot arise */
 	{
 	  fprintf(stderr, "add_segment: error\n");
 	  break;
@@ -615,9 +597,9 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
       /* two resulting trapezoids t and tn as the upper neighbours of */
       /* the sole lower trapezoid */
 
-      else if ((tr[t].d0 > 0) && (tr[t].d1 <= 0))
+      else if (tr[t].d0 > 0 && tr[t].d1 <= 0)
 	{			/* Only one trapezoid below */
-	  if ((tr[t].u0 > 0) && (tr[t].u1 > 0))
+	  if (tr[t].u0 > 0 && tr[t].u1 > 0)
 	    {			/* continuation of a chain from abv. */
 	      if (tr[t].usave > 0) /* three upper neighbours */
 		{
@@ -656,11 +638,9 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
 	    {			/* fresh seg. or upward cusp */
 	      int tmp_u = tr[t].u0;
 	      int td0, td1;
-	      if (((td0 = tr[tmp_u].d0) > 0) &&
-		  ((td1 = tr[tmp_u].d1) > 0))
+	      if ((td0 = tr[tmp_u].d0) > 0 && (td1 = tr[tmp_u].d1) > 0)
 		{		/* upward cusp */
-		  if ((tr[td0].rseg > 0) &&
-		      !is_left_of(tr[td0].rseg, seg, &s.v1))
+		  if (tr[td0].rseg > 0 && !is_left_of(tr[td0].rseg, seg, &s.v1))
 		    {
 		      tr[t].u0 = tr[t].u1 = tr[tn].u1 = -1;
 		      tr[tr[tn].u0].d1 = tn;
@@ -687,7 +667,7 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
 	      else
 		tmptriseg = seg[segnum].next;
 
-	      if ((tmptriseg > 0) && is_left_of(tmptriseg, seg, &s.v0))
+	      if (tmptriseg > 0 && is_left_of(tmptriseg, seg, &s.v0))
 		{
 				/* L-R downward cusp */
 		  tr[tr[t].d0].u0 = t;
@@ -702,7 +682,7 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
 	    }
 	  else
 	    {
-	      if ((tr[tr[t].d0].u0 > 0) && (tr[tr[t].d0].u1 > 0))
+	      if (tr[tr[t].d0].u0 > 0 && tr[tr[t].d0].u1 > 0)
 		{
 		  if (tr[tr[t].d0].u0 == t) /* passes through LHS */
 		    {
@@ -723,9 +703,9 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
 	}
 
 
-      else if ((tr[t].d0 <= 0) && (tr[t].d1 > 0))
+      else if (tr[t].d0 <= 0 && tr[t].d1 > 0)
 	{			/* Only one trapezoid below */
-	  if ((tr[t].u0 > 0) && (tr[t].u1 > 0))
+	  if (tr[t].u0 > 0 && tr[t].u1 > 0)
 	    {			/* continuation of a chain from abv. */
 	      if (tr[t].usave > 0) /* three upper neighbours */
 		{
@@ -764,11 +744,9 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
 	    {			/* fresh seg. or upward cusp */
 	      int tmp_u = tr[t].u0;
 	      int td0, td1;
-	      if (((td0 = tr[tmp_u].d0) > 0) &&
-		  ((td1 = tr[tmp_u].d1) > 0))
+	      if ((td0 = tr[tmp_u].d0) > 0 && (td1 = tr[tmp_u].d1) > 0)
 		{		/* upward cusp */
-		  if ((tr[td0].rseg > 0) &&
-		      !is_left_of(tr[td0].rseg, seg, &s.v1))
+		  if (tr[td0].rseg > 0 && !is_left_of(tr[td0].rseg, seg, &s.v1))
 		    {
 		      tr[t].u0 = tr[t].u1 = tr[tn].u1 = -1;
 		      tr[tr[tn].u0].d1 = tn;
@@ -789,15 +767,13 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
 	  if (FP_EQUAL(tr[t].lo.y, tr[tlast].lo.y) &&
 	      FP_EQUAL(tr[t].lo.x, tr[tlast].lo.x) && tribot)
 	    {		/* bottom forms a triangle */
-	      /* int tmpseg; */
 
 	      if (is_swapped)
 		tmptriseg = seg[segnum].prev;
 	      else
 		tmptriseg = seg[segnum].next;
 
-	      /* if ((tmpseg > 0) && is_left_of(tmpseg, seg, &s.v0)) */
-	      if ((tmptriseg > 0) && is_left_of(tmptriseg, seg, &s.v0))
+	      if (tmptriseg > 0 && is_left_of(tmptriseg, seg, &s.v0))
 		{
 		  /* L-R downward cusp */
 		  tr[tr[t].d1].u0 = t;
@@ -812,7 +788,7 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
 	    }
 	  else
 	    {
-	      if ((tr[tr[t].d1].u0 > 0) && (tr[tr[t].d1].u1 > 0))
+	      if (tr[tr[t].d1].u0 > 0 && tr[tr[t].d1].u1 > 0)
 		{
 		  if (tr[tr[t].d1].u0 == t) /* passes through LHS */
 		    {
@@ -837,7 +813,6 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
 
       else
 	{
-	  /* int tmpseg = tr[tr[t].d0].rseg; */
 	  double y0, yt;
 	  pointf tmppt;
 	  int tnext, i_d0, i_d1;
@@ -865,7 +840,7 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
 	  /* check continuity from the top so that the lower-neighbour */
 	  /* values are properly filled for the upper trapezoid */
 
-	  if ((tr[t].u0 > 0) && (tr[t].u1 > 0))
+	  if (tr[t].u0 > 0 && tr[t].u1 > 0)
 	    {			/* continuation of a chain from abv. */
 	      if (tr[t].usave > 0) /* three upper neighbours */
 		{
@@ -905,11 +880,9 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
 	    {			/* fresh seg. or upward cusp */
 	      int tmp_u = tr[t].u0;
 	      int td0, td1;
-	      if (((td0 = tr[tmp_u].d0) > 0) &&
-		  ((td1 = tr[tmp_u].d1) > 0))
+	      if ((td0 = tr[tmp_u].d0) > 0 && (td1 = tr[tmp_u].d1) > 0)
 		{		/* upward cusp */
-		  if ((tr[td0].rseg > 0) &&
-		      !is_left_of(tr[td0].rseg, seg, &s.v1))
+		  if (tr[td0].rseg > 0 && !is_left_of(tr[td0].rseg, seg, &s.v1))
 		    {
 		      tr[t].u0 = tr[t].u1 = tr[tn].u1 = -1;
 		      tr[tr[tn].u0].d1 = tn;
@@ -991,7 +964,7 @@ add_segment (int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
   merge_trapezoids(segnum, tfirstl, tlastl, S_LEFT, tr, qs);
   merge_trapezoids(segnum, tfirstr, tlastr, S_RIGHT, tr, qs);
 
-  seg[segnum].is_inserted = TRUE;
+  seg[segnum].is_inserted = true;
   return 0;
 }
 
@@ -1016,21 +989,21 @@ find_new_roots(int segnum, segment_t* seg, trap_t* tr, qnode_t* qs)
 /* Get log*n for given n */
 static int math_logstar_n(int n)
 {
-  register int i;
+  int i;
   double v;
 
   for (i = 0, v = (double) n; v >= 1; i++)
       v = log2(v);
 
-  return (i - 1);
+  return i - 1;
 }
 
 static int math_N(int n, int h)
 {
-  register int i;
+  int i;
   double v;
 
-  for (i = 0, v = (int) n; i < h; i++)
+  for (i = 0, v = (double) n; i < h; i++)
       v = log2(v);
 
   return (int) ceil((double) 1.0*n/v);
@@ -1050,7 +1023,7 @@ construct_trapezoids(int nseg, segment_t* seg, int* permute, int ntraps,
     TRSIZE = ntraps;
     qs = N_NEW (2*ntraps, qnode_t);
     q_idx = tr_idx = 1;
-    memset((void *)tr, 0, ntraps*sizeof(trap_t));
+    memset(tr, 0, ntraps*sizeof(trap_t));
 
   /* Add the first segment and get the query structure and trapezoid */
   /* list initialised */
