@@ -92,13 +92,15 @@ static void svg_print_id_class(GVJ_t * job, char* id, char* idx, char* kind, voi
     char* str;
 
     GVPUTS(job, "<g id=\"");
-    gvputs(job, xml_string(id));
-    if (idx)
-	gvprintf (job, "_%s", xml_string(idx));
+    gvputs_xml(job, id);
+    if (idx) {
+	GVPUTS(job, "_");
+	gvputs_xml(job, idx);
+    }
     gvprintf(job, "\" class=\"%s", kind);
     if ((str = agget(obj, "class")) && *str) {
 	GVPUTS(job, " ");
-	gvputs(job, xml_string(str));
+	gvputs_xml(job, str);
     }
     GVPUTS(job, "\"");
 }
@@ -162,7 +164,7 @@ static void svg_grstyle(GVJ_t * job, int filled, int gid)
 static void svg_comment(GVJ_t * job, char *str)
 {
     GVPUTS(job, "<!-- ");
-    gvputs(job, xml_string(str));
+    gvputs_xml(job, str);
     GVPUTS(job, " -->\n");
 }
 
@@ -180,11 +182,11 @@ static void svg_begin_job(GVJ_t * job)
                 " \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n"
 
                 "<!-- Generated by ");
-    gvputs(job, xml_string(job->common->info[0]));
+    gvputs_xml(job, job->common->info[0]);
     GVPUTS(job, " version ");
-    gvputs(job, xml_string(job->common->info[1]));
+    gvputs_xml(job, job->common->info[1]);
     GVPUTS(job, " (");
-    gvputs(job, xml_string(job->common->info[2]));
+    gvputs_xml(job, job->common->info[2]);
     GVPUTS(job, ")\n"
                 " -->\n");
 }
@@ -196,7 +198,7 @@ static void svg_begin_graph(GVJ_t * job)
     GVPUTS(job, "<!--");
     if (agnameof(obj->u.g)[0] && agnameof(obj->u.g)[0] != LOCALNAMEPREFIX) {
 	GVPUTS(job, " Title: ");
-	gvputs(job, xml_string(agnameof(obj->u.g)));
+	gvputs_xml(job, agnameof(obj->u.g));
     }
     gvprintf(job, " Pages: %d -->\n",
 	     job->pagesArraySize.x * job->pagesArraySize.y);
@@ -249,9 +251,8 @@ static void svg_begin_page(GVJ_t * job)
      * and it is the entire graph if we're not currently paging */
     svg_print_id_class(job, obj->id, NULL, "graph", obj->u.g);
     GVPUTS(job, " transform=\"scale(");
-    gvprintdouble(job, job->scale.x);
-    GVPUTS(job, " ");
-    gvprintdouble(job, job->scale.y);
+    // cannot be gvprintdouble because 2 digits precision insufficient
+    gvprintf(job, "%g %g", job->scale.x, job->scale.y);
     gvprintf(job, ") rotate(%d) translate(", -job->rotation);
     gvprintdouble(job, job->translation.x);
     GVPUTS(job, " ");
@@ -260,7 +261,7 @@ static void svg_begin_page(GVJ_t * job)
     /* default style */
     if (agnameof(obj->u.g)[0] && agnameof(obj->u.g)[0] != LOCALNAMEPREFIX) {
 	GVPUTS(job, "<title>");
-	gvputs(job, xml_string(agnameof(obj->u.g)));
+	gvputs_xml(job, agnameof(obj->u.g));
 	GVPUTS(job, "</title>\n");
     }
 }
@@ -277,7 +278,7 @@ static void svg_begin_cluster(GVJ_t * job)
     svg_print_id_class(job, obj->id, NULL, "cluster", obj->u.sg);
     GVPUTS(job, ">\n"
                 "<title>");
-    gvputs(job, xml_string(agnameof(obj->u.g)));
+    gvputs_xml(job, agnameof(obj->u.g));
     GVPUTS(job, "</title>\n");
 }
 
@@ -298,7 +299,7 @@ static void svg_begin_node(GVJ_t * job)
     svg_print_id_class(job, obj->id, idx, "node", obj->u.n);
     GVPUTS(job, ">\n"
                 "<title>");
-    gvputs(job, xml_string(agnameof(obj->u.n)));
+    gvputs_xml(job, agnameof(obj->u.n));
     GVPUTS(job, "</title>\n");
 }
 
@@ -316,8 +317,8 @@ static void svg_begin_edge(GVJ_t * job)
     GVPUTS(job, ">\n"
 
                 "<title>");
-    ename = strdup_and_subst_obj("\\E", (void *) (obj->u.e));
-    gvputs(job, xml_string(ename));
+    ename = strdup_and_subst_obj("\\E", obj->u.e);
+    gvputs_xml(job, ename);
     free(ename);
     GVPUTS(job, "</title>\n");
 }
@@ -334,7 +335,7 @@ svg_begin_anchor(GVJ_t * job, char *href, char *tooltip, char *target,
     GVPUTS(job, "<g");
     if (id) {
 	GVPUTS(job, " id=\"a_");
-        gvputs(job, xml_string(id));
+        gvputs_xml(job, id);
         GVPUTS(job, "\"");
     }
     GVPUTS(job, ">"
@@ -342,17 +343,19 @@ svg_begin_anchor(GVJ_t * job, char *href, char *tooltip, char *target,
                 "<a");
     if (href && href[0]) {
 	GVPUTS(job, " xlink:href=\"");
-	gvputs(job, xml_url_string(href));
+	const xml_flags_t flags = {0};
+	xml_escape(href, flags, (int(*)(void*, const char*))gvputs, job);
 	GVPUTS(job, "\"");
     }
     if (tooltip && tooltip[0]) {
 	GVPUTS(job, " xlink:title=\"");
-	gvputs(job, xml_string0(tooltip, 1));
+	const xml_flags_t flags = {.raw = 1, .dash = 1, .nbsp = 1};
+	xml_escape(tooltip, flags, (int(*)(void*, const char*))gvputs, job);
 	GVPUTS(job, "\"");
     }
     if (target && target[0]) {
 	GVPUTS(job, " target=\"");
-	gvputs(job, xml_string(target));
+	gvputs_xml(job, target);
 	GVPUTS(job, "\"");
     }
     GVPUTS(job, ">\n");
@@ -470,12 +473,14 @@ static void svg_textspan(GVJ_t * job, pointf p, textspan_t * span)
     }
     GVPUTS(job, ">");
     if (obj->labeledgealigned) {
-	gvprintf(job, "<textPath xlink:href=\"#%s_p\" startOffset=\"50%%\">", xml_string(obj->id));
-	GVPUTS(job, "<tspan x=\"0\" dy=\"");
+	GVPUTS(job, "<textPath xlink:href=\"#");
+	gvputs_xml(job, obj->id);
+	GVPUTS(job, "_p\" startOffset=\"50%\"><tspan x=\"0\" dy=\"");
         gvprintdouble(job, -p.y);
         GVPUTS(job, "\">");
     }
-    gvputs(job, xml_string0(span->str, TRUE));
+    const xml_flags_t xml_flags = {.raw = 1, .dash = 1, .nbsp = 1};
+    xml_escape(span->str, xml_flags, (int(*)(void*, const char*))gvputs, job);
     if (obj->labeledgealigned)
 	GVPUTS(job, "</tspan></textPath>");
     GVPUTS(job, "</text>\n");
@@ -487,12 +492,11 @@ static void svg_textspan(GVJ_t * job, pointf p, textspan_t * span)
 static int svg_gradstyle(GVJ_t * job, pointf * A, int n)
 {
     pointf G[2];
-    float angle;
     static int gradId;
     int id = gradId++;
 
     obj_state_t *obj = job->obj;
-    angle = obj->gradient_angle * M_PI / 180;	//angle of gradient line
+    double angle = obj->gradient_angle * M_PI / 180; //angle of gradient line
     G[0].x = G[0].y = G[1].x = G[1].y = 0.;
     get_gradient_points(A, G, n, angle, 0);	//get points on gradient line
 
@@ -618,7 +622,7 @@ svg_bezier(GVJ_t * job, pointf * A, int n, int arrow_at_start,
     GVPUTS(job, "<path");
     if (obj->labeledgealigned) {
 	GVPUTS(job, " id=\"");
-	gvputs(job, xml_string(obj->id));
+	gvputs_xml(job, obj->id);
 	GVPUTS(job, "_p\" ");
     } 
     svg_grstyle(job, filled, gid);
