@@ -10,6 +10,7 @@
 
 #include "config.h"
 
+#include <stdbool.h>
 #include <stdint.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -36,10 +37,8 @@ attr_value_edited_cb(GtkCellRendererText *renderer, gchar *pathStr, gchar *newTe
 	GtkTreePath *path;
 	GtkTreeIter iter;
 	gchar *old_attr;
-	gint row;
 	
 	path = gtk_tree_path_new_from_string(pathStr);
-	row = gtk_tree_path_get_indices(path)[0];
 	
 	// need to free old attr value in job and allocate new attr value - how?
 	
@@ -72,7 +71,7 @@ static void gtk_initialize(GVJ_t *firstjob)
     scr = DefaultScreen(dpy);
     firstjob->device_dpi.x = DisplayWidth(dpy, scr) * 25.4 / DisplayWidthMM(dpy, scr);
     firstjob->device_dpi.y = DisplayHeight(dpy, scr) * 25.4 / DisplayHeightMM(dpy, scr);
-    firstjob->device_sets_dpi = TRUE;
+    firstjob->device_sets_dpi = true;
 }
 
 static void gtk_finalize(GVJ_t *firstjob)
