@@ -10,6 +10,7 @@
 
 #include "config.h"
 
+#include <stdbool.h>
 #include <stdlib.h>
 #include <sys/stat.h>
 
@@ -30,7 +31,7 @@ typedef enum {
 
 static void gvloadimage_rsvg_free(usershape_t *us)
 {
-    rsvg_handle_close((RsvgHandle*)us->data, NULL);
+    rsvg_handle_close(us->data, NULL);
 }
 
 static RsvgHandle* gvloadimage_rsvg_load(GVJ_t * job, usershape_t *us)
@@ -50,7 +51,7 @@ static RsvgHandle* gvloadimage_rsvg_load(GVJ_t * job, usershape_t *us)
 
     if (us->data) {
         if (us->datafree == gvloadimage_rsvg_free)
-             rsvgh = (RsvgHandle*)(us->data); /* use cached data */
+             rsvgh = us->data; /* use cached data */
         else {
              us->datafree(us);        /* free incompatible cache data */
              us->data = NULL;
@@ -158,7 +159,7 @@ static RsvgHandle* gvloadimage_rsvg_load(GVJ_t * job, usershape_t *us)
         }
 
         if (rsvgh) {
-            us->data = (void*)rsvgh;
+            us->data = rsvgh;
             us->datafree = gvloadimage_rsvg_free;
         }
 
@@ -168,7 +169,7 @@ static RsvgHandle* gvloadimage_rsvg_load(GVJ_t * job, usershape_t *us)
     return rsvgh;
 }
 
-static void gvloadimage_rsvg_cairo(GVJ_t * job, usershape_t *us, boxf b, boolean filled)
+static void gvloadimage_rsvg_cairo(GVJ_t * job, usershape_t *us, boxf b, bool filled)
 {
     RsvgHandle* rsvgh = gvloadimage_rsvg_load(job, us);
 
