@@ -5,7 +5,7 @@ from pathlib import Path
 import sys
 
 sys.path.append(os.path.dirname(__file__))
-from gvtest import run_c
+from gvtest import run_c # pylint: disable=wrong-import-position
 
 def test_itos():
   """run the itos unit tests"""
@@ -20,6 +20,4 @@ def test_itos():
   # extra C flags this compilation needs
   cflags = ['-I', lib]
 
-  ret, _, _ = run_c(src, cflags=cflags)
-
-  assert ret == 0
+  _, _ = run_c(src, cflags=cflags)
