@@ -15,13 +15,21 @@ import signal
 import stat
 import subprocess
 import sys
+import sysconfig
 import tempfile
 from typing import List
 import xml.etree.ElementTree as ET
 import pytest
 
 sys.path.append(os.path.dirname(__file__))
-from gvtest import ROOT, run_c #pylint: disable=C0413
+from gvtest import dot, ROOT, run_c #pylint: disable=C0413
+
+def is_mingw() -> bool:
+  """
+  are we running on MinGW?
+  """
+
+  return "mingw" in sysconfig.get_platform()
 
 def is_ndebug_defined() -> bool:
   """
@@ -34,6 +42,18 @@ def is_ndebug_defined() -> bool:
 
   return False
 
+def remove_xtype_warnings(s: str) -> str:
+  """
+  Remove macOS XType warnings from a string. These appear to be harmless, but
+  occur in CI.
+  """
+
+  # avoid doing this anywhere except on macOS
+  if platform.system() != "Darwin":
+    return s
+
+  return re.sub(r"^.* XType: .*\.$", "", s, flags=re.MULTILINE)
+
 # The terminology used in rtest.py is a little inconsistent. At the
 # end it reports the total number of tests, the number of "failures"
 # (crashes) and the number of "changes" (which is the number of tests
@@ -59,9 +79,9 @@ def test_regression_failure():
   """
 
   os.chdir(Path(__file__).resolve().parent)
-  result = subprocess.Popen([sys.executable, "rtest.py"],
-                            stderr=subprocess.PIPE, universal_newlines=True)
-  text = result.communicate()[1]
+  with subprocess.Popen([sys.executable, "rtest.py"], stderr=subprocess.PIPE,
+                        universal_newlines=True) as result:
+    text = result.communicate()[1]
   print(text)
   assert "Layout failures: 0" in text
 # FIXME: re-enable when all tests pass on all platforms
@@ -79,7 +99,7 @@ def test_14():
   assert input.exists(), "unexpectedly missing test case"
 
   # process it with Graphviz
-  subprocess.check_call(["dot", "-Tsvg", "-o", os.devnull, input])
+  dot("svg", input)
 
 def test_56():
   """
@@ -95,11 +115,11 @@ def test_56():
   # FIXME: remove this block when this #56 is fixed
   if not is_ndebug_defined() and platform.system() != "Windows":
     with pytest.raises(subprocess.CalledProcessError):
-      subprocess.check_call(["dot", "-Tsvg", "-o", os.devnull, input])
+      dot("svg", input)
     return
 
   # process it with Graphviz
-  subprocess.check_call(["dot", "-Tsvg", "-o", os.devnull, input])
+  dot("svg", input)
 
 def test_131():
   """
@@ -108,11 +128,10 @@ def test_131():
   """
 
   # a basic graph
-  dot = "digraph { a -> b; c -> d; }"
+  src = "digraph { a -> b; c -> d; }"
 
   # ask Graphviz to process this to PIC
-  pic = subprocess.check_output(["dot", "-Tpic"], input=dot,
-    universal_newlines=True)
+  pic = dot("pic", source=src)
 
   if shutil.which("gpic") is None:
     pytest.skip("GNU PIC not available")
@@ -133,8 +152,7 @@ def test_144(testcase: str):
   assert input.exists(), "unexpectedly missing test case"
 
   # process the non-ortho one into JSON
-  out = subprocess.check_output(["dot", "-Tjson", input],
-    universal_newlines=True)
+  out = dot("json", input)
   data = json.loads(out)
 
   # find the two nodes, “A” and “B”
@@ -183,8 +201,7 @@ def test_165():
   assert input.exists(), "unexpectedly missing test case"
 
   # ask Graphviz to translate it to xdot
-  output = subprocess.check_output(["dot", "-Txdot", input],
-    universal_newlines=True)
+  output = dot("xdot", input)
 
   # find the line containing the _ldraw_ attribute
   ldraw = re.search(r"^\s*_ldraw_\s*=(?P<value>.*?)$", output, re.MULTILINE)
@@ -205,8 +222,7 @@ def test_165_2():
   assert input.exists(), "unexpectedly missing test case"
 
   # ask Graphviz to translate it to xdot
-  output = subprocess.check_output(["dot", "-Txdot", input],
-    universal_newlines=True)
+  output = dot("xdot", input)
 
   # find the lines containing _ldraw_ attributes
   ldraw = re.findall(r"^\s*_ldraw_\s*=(.*?)$", output, re.MULTILINE)
@@ -227,8 +243,7 @@ def test_165_3():
   assert input.exists(), "unexpectedly missing test case"
 
   # ask Graphviz to translate it to xdot
-  output = subprocess.check_output(["dot", "-Txdot", input],
-    universal_newlines=True)
+  output = dot("xdot", input)
 
   # find the lines containing _ldraw_ attributes
   ldraw = re.findall(r"^\s*_ldraw_\s*=(.*?)$", output, re.MULTILINE)
@@ -254,6 +269,26 @@ def test_167():
   # Graphviz should not have caused a segfault
   assert ret != -signal.SIGSEGV, "Graphviz segfaulted"
 
+def test_191():
+  """
+  a comma-separated list without quotes should cause a hard error, not a warning
+  https://gitlab.com/graphviz/graphviz/-/issues/191
+  """
+
+  source = 'graph {\n' \
+           '  "Trackable" [fontcolor=grey45,labelloc=c,fontname=Vera Sans, ' \
+           'DejaVu Sans, Liberation Sans, Arial, Helvetica, sans,shape=box,' \
+           'height=0.3,align=center,fontsize=10,style="setlinewidth(0.5)"];\n' \
+           '}'
+
+  with subprocess.Popen(["dot", "-Tdot"], stdin=subprocess.PIPE,
+                        stderr=subprocess.PIPE, universal_newlines=True) as p:
+    _, stderr = p.communicate(source)
+
+    assert "syntax error" in stderr, "missing error message for unquoted list"
+
+    assert p.returncode != 0, "syntax error was only a warning, not an error"
+
 @pytest.mark.skipif(shutil.which("gv2gxl") is None or
                     shutil.which("gxl2gv") is None,
                     reason="GXL tools not available")
@@ -271,24 +306,20 @@ def test_517():
     '}'
 
   # translate it to GXL
-  p = subprocess.Popen(["gv2gxl"], stdin=subprocess.PIPE,
-    stdout=subprocess.PIPE, universal_newlines=True)
-  gxl, _ = p.communicate(input)
-  assert p.returncode == 0
+  gxl = subprocess.check_output(["gv2gxl"], input=input,
+    universal_newlines=True)
 
   # translate this back to Dot
-  p = subprocess.Popen(["gxl2gv"], stdin=subprocess.PIPE,
-    stdout=subprocess.PIPE, universal_newlines=True)
-  dot, _ = p.communicate(gxl)
-  assert p.returncode == 0
+  dot_output = subprocess.check_output(["gxl2gv"], input=gxl,
+    universal_newlines=True)
 
   # the result should have both expected labels somewhere
   assert \
-    "label=<<TABLE><TR><TD>(</TD><TD>A</TD><TD>)</TD></TR></TABLE>>" in dot, \
-    "HTML label missing"
+    "label=<<TABLE><TR><TD>(</TD><TD>A</TD><TD>)</TD></TR></TABLE>>" in \
+    dot_output, "HTML label missing"
   assert \
-    'label="<TABLE><TR><TD>(</TD><TD>B</TD><TD>)</TD></TR></TABLE>"' in dot, \
-    "regular label missing"
+    'label="<TABLE><TR><TD>(</TD><TD>B</TD><TD>)</TD></TR></TABLE>"' in \
+    dot_output, "regular label missing"
 
 def test_793():
   """
@@ -304,11 +335,11 @@ def test_793():
 
     # ask the VRML back end to handle a simple graph, using the above as the
     # current working directory
-    p = subprocess.Popen(["dot", "-Tvrml", "-o", os.devnull], cwd=t)
-    p.communicate("digraph { a -> b; }")
+    with subprocess.Popen(["dot", "-Tvrml", "-o", os.devnull], cwd=t) as p:
+      p.communicate("digraph { a -> b; }")
 
-  # Graphviz should not have caused a segfault
-  assert p.returncode != -signal.SIGSEGV, "Graphviz segfaulted"
+      # Graphviz should not have caused a segfault
+      assert p.returncode != -signal.SIGSEGV, "Graphviz segfaulted"
 
 def test_797():
   """
@@ -322,11 +353,7 @@ def test_797():
           '}'
 
   # process this with the client-side imagemap back end
-  p = subprocess.Popen(["dot", "-Tcmapx"], stdin=subprocess.PIPE,
-    stdout=subprocess.PIPE, universal_newlines=True)
-  output, _ = p.communicate(input)
-
-  assert p.returncode == 0
+  output = dot("cmapx", source=input)
 
   # the escape sequences should have been preserved
   assert "&amp; &amp;" in output
@@ -342,7 +369,7 @@ def test_1221():
   assert input.exists(), "unexpectedly missing test case"
 
   # process this with dot
-  subprocess.check_call(["dot", "-Tsvg", "-o", os.devnull, input])
+  dot("svg", input)
 
 @pytest.mark.skipif(shutil.which("gv2gml") is None,
                     reason="gv2gml not available")
@@ -353,16 +380,12 @@ def test_1276():
   """
 
   # DOT input containing a label with quotes
-  dot = 'digraph test {\n' \
+  src = 'digraph test {\n' \
         '  x[label=<"Label">];\n' \
         '}'
 
   # process this to GML
-  p = subprocess.Popen(["gv2gml"], stdin=subprocess.PIPE,
-                       stdout=subprocess.PIPE, universal_newlines=True)
-  gml, _ = p.communicate(dot)
-
-  assert p.returncode == 0, "gv2gml failed"
+  gml = subprocess.check_output(["gv2gml"], input=src, universal_newlines=True)
 
   # the unescaped label should not appear in the output
   assert '""Label""' not in gml, "quotes not escaped in label"
@@ -383,14 +406,14 @@ def test_1314():
 
   # ask Graphviz to process it, which should fail
   try:
-    subprocess.check_call(["dot", "-Tsvg", "-o", os.devnull, input])
+    dot("svg", input)
   except subprocess.CalledProcessError:
     return
 
   # the execution did not fail as expected
   pytest.fail("dot incorrectly exited with success")
 
-@pytest.mark.xfail(strict=not is_ndebug_defined()) # FIXME
+@pytest.mark.xfail(strict=not is_ndebug_defined() and not is_mingw()) # FIXME
 def test_1408():
   """
   parsing particular ortho layouts should not cause an assertion failure
@@ -402,7 +425,7 @@ def test_1408():
   assert input.exists(), "unexpectedly missing test case"
 
   # process it with Graphviz
-  subprocess.check_call(["dot", "-Tsvg", "-o", os.devnull, input])
+  dot("svg", input)
 
 def test_1411():
   """
@@ -415,11 +438,11 @@ def test_1411():
   assert input.exists(), "unexpectedly missing test case"
 
   # process it with Graphviz (should fail)
-  p = subprocess.Popen(["dot", "-Tsvg", "-o", os.devnull, input],
-    stderr=subprocess.PIPE, universal_newlines=True)
-  _, output = p.communicate()
+  with subprocess.Popen(["dot", "-Tsvg", "-o", os.devnull, input],
+                        stderr=subprocess.PIPE, universal_newlines=True) as p:
+    _, output = p.communicate()
 
-  assert p.returncode != 0, "Graphviz accepted broken input"
+    assert p.returncode != 0, "Graphviz accepted broken input"
 
   assert "syntax error in line 17 near '\\'" in output, \
     'error message did not identify correct location'
@@ -436,7 +459,7 @@ def test_1436():
 
   # ask Graphviz to process it, which should generate a segfault if this bug
   # has been reintroduced
-  subprocess.check_call(["dot", "-Tsvg", "-o", os.devnull, input])
+  dot("svg", input)
 
 def test_1444():
   """
@@ -450,26 +473,28 @@ def test_1444():
   assert input1.exists(), "unexpectedly missing test case"
 
   # ask Graphviz to process it
-  p = subprocess.Popen(["dot", "-Tsvg", input1], stdout=subprocess.PIPE,
-    stderr=subprocess.PIPE, universal_newlines=True)
-  stdout1, stderr = p.communicate()
+  with subprocess.Popen(["dot", "-Tsvg", input1], stdout=subprocess.PIPE,
+                        stderr=subprocess.PIPE, universal_newlines=True) as p:
+    stdout1, stderr = p.communicate()
 
-  assert p.returncode == 0, "failed to process a headport edge"
+    assert p.returncode == 0, "failed to process a headport edge"
 
-  assert stderr.strip() == "", "emitted an error for a legal graph"
+  stderr = remove_xtype_warnings(stderr).strip()
+  assert stderr == "", "emitted an error for a legal graph"
 
   # now locate our second variant, that simply has the attributes swapped
   input2 = Path(__file__).parent / "1444-2.dot"
   assert input2.exists(), "unexpectedly missing test case"
 
   # process it identically
-  p = subprocess.Popen(["dot", "-Tsvg", input2], stdout=subprocess.PIPE,
-    stderr=subprocess.PIPE, universal_newlines=True)
-  stdout2, stderr = p.communicate()
+  with subprocess.Popen(["dot", "-Tsvg", input2], stdout=subprocess.PIPE,
+                        stderr=subprocess.PIPE, universal_newlines=True) as p:
+    stdout2, stderr = p.communicate()
 
-  assert p.returncode == 0, "failed to process a headport edge"
+    assert p.returncode == 0, "failed to process a headport edge"
 
-  assert stderr.strip() == "", "emitted an error for a legal graph"
+  stderr = remove_xtype_warnings(stderr).strip()
+  assert stderr == "", "emitted an error for a legal graph"
 
   assert stdout1 == stdout2, \
     "swapping edge attributes altered the output graph"
@@ -481,14 +506,14 @@ def test_1449():
   """
 
   # start Graphviz
-  p = subprocess.Popen(["dot", "-Tsvg", "-o", os.devnull],
-    stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
-    universal_newlines=True)
+  with subprocess.Popen(["dot", "-Tsvg", "-o", os.devnull],
+                        stdin=subprocess.PIPE,
+                        stderr=subprocess.PIPE, universal_newlines=True) as p:
 
-  # pass it some input that uses the SVG color scheme
-  _, stderr = p.communicate('graph g { colorscheme="svg"; }')
+    # pass it some input that uses the SVG color scheme
+    _, stderr = p.communicate('graph g { colorscheme="svg"; }')
 
-  assert p.returncode == 0, "Graphviz exited with non-zero status"
+    assert p.returncode == 0, "Graphviz exited with non-zero status"
 
   assert stderr.strip() == "", "SVG color scheme use caused warnings"
 
@@ -500,18 +525,15 @@ def test_1594():
   """
 
   # locate our associated test case in this directory
-# FIXME: remove cwd workaround when
-# https://gitlab.com/graphviz/graphviz/-/issues/1780 is fixed
-#    input = Path(__file__).parent / "1594.gvpr"
-  input = "1594.gvpr"
+  input = Path(__file__).parent / "1594.gvpr"
 
   # run GVPR with our (malformed) input program
-  p = subprocess.Popen(["gvpr", "-f", input], stdin=subprocess.PIPE,
-    cwd=os.path.join(os.path.dirname(__file__)),
-    stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
-  _, stderr = p.communicate()
+  with subprocess.Popen(["gvpr", "-f", input], stdin=subprocess.PIPE,
+                        stdout=subprocess.DEVNULL, stderr=subprocess.PIPE,
+                        universal_newlines=True) as p:
+    _, stderr = p.communicate()
 
-  assert p.returncode != 0, "GVPR did not reject malformed program"
+    assert p.returncode != 0, "GVPR did not reject malformed program"
 
   assert "line 3:" in stderr, \
     "GVPR did not identify correct line of syntax error"
@@ -528,7 +550,7 @@ def test_1658():
   assert input.exists(), "unexpectedly missing test case"
 
   # process it with Graphviz
-  subprocess.check_call(["dot", "-Tpng", "-o", os.devnull, input])
+  dot("png", input)
 
 def test_1676():
   """
@@ -576,11 +598,10 @@ def test_1767():
   assert c_src.exists(), "missing test case"
 
   # find our co-located dot input
-  dot = (Path(__file__).parent / "1767.dot").resolve()
-  assert dot.exists(), "missing test case"
+  src = (Path(__file__).parent / "1767.dot").resolve()
+  assert src.exists(), "missing test case"
 
-  ret, _, _ = run_c(c_src, [dot], link=["cgraph", "gvc"])
-  assert ret == 0
+  _, _ = run_c(c_src, [src], link=["cgraph", "gvc"])
 
   # FIXME: uncomment this when #1767 is fixed
   # assert stdout == "Loaded graph:clusters\n" \
@@ -641,19 +662,6 @@ def test_1813():
 
   assert "Usage" in output, "gvedit -? did not show usage"
 
-@pytest.mark.skipif(shutil.which("lefty") is None, reason="Lefty not available")
-def test_1818():
-  """
-  lefty -? should show usage
-  https://gitlab.com/graphviz/graphviz/-/issues/1813
-  """
-
-  output = subprocess.check_output(["lefty", "-?"],
-    stderr=subprocess.STDOUT,
-    universal_newlines=True)
-
-  assert "Usage" in output, "lefty -? did not show usage"
-
 def test_1845():
   """
   rendering sequential graphs to PS should not segfault
@@ -665,7 +673,7 @@ def test_1845():
   assert input.exists(), "unexpectedly missing test case"
 
   # generate a multipage PS file from this input
-  subprocess.check_call(["dot", "-Tps", "-o", os.devnull, input])
+  dot("ps", input)
 
 @pytest.mark.xfail(strict=True) # FIXME
 def test_1856():
@@ -679,8 +687,7 @@ def test_1856():
   assert input.exists(), "unexpectedly missing test case"
 
   # process it into JSON
-  out = subprocess.check_output(["dot", "-Tjson", input],
-    universal_newlines=True)
+  out = dot("json", input)
   data = json.loads(out)
 
   # find the two nodes, “3” and “5”
@@ -747,11 +754,11 @@ def test_1876():
   input = "graph { a }"
 
   # process this with fdp
-  p = subprocess.Popen(["fdp"], stdin=subprocess.PIPE, stdout=subprocess.PIPE,
-    universal_newlines=True)
-  output, _ = p.communicate(input)
-
-  assert p.returncode == 0, "fdp failed to process trivial graph"
+  try:
+    output = subprocess.check_output(["fdp"], input=input,
+      universal_newlines=True)
+  except subprocess.CalledProcessError as e:
+    raise RuntimeError("fdp failed to process trivial graph") from e
 
   # we should not see any internal names like "%3"
   assert "%" not in output, "internal name in fdp output"
@@ -767,10 +774,8 @@ def test_1877():
   input = "graph {subgraph cluster_a {}; cluster_a -- b}"
 
   # fdp should be able to process this
-  p = subprocess.Popen(["fdp", "-o", os.devnull], stdin=subprocess.PIPE,
+  subprocess.run(["fdp", "-o", os.devnull], input=input, check=True,
     universal_newlines=True)
-  p.communicate(input)
-  assert p.returncode == 0
 
 def test_1880():
   """
@@ -786,11 +791,11 @@ def test_1880():
   # FIXME: remove this block when this #1880 is fixed
   if not is_ndebug_defined() and platform.system() != "Windows":
     with pytest.raises(subprocess.CalledProcessError):
-      subprocess.check_call(["dot", "-Tpng", "-o", os.devnull, input])
+      dot("png", input)
     return
 
   # process it with Graphviz
-  subprocess.check_call(["dot", "-Tpng", "-o", os.devnull, input])
+  dot("png", input)
 
 def test_1898():
   """
@@ -804,7 +809,21 @@ def test_1898():
 
   # ask Graphviz to process it, which should generate a segfault if this bug
   # has been reintroduced
-  subprocess.check_call(["dot", "-Tsvg", "-o", os.devnull, input])
+  dot("svg", input)
+
+def test_1902():
+  """
+  test a segfault from https://gitlab.com/graphviz/graphviz/-/issues/1902 has
+  not reappeared
+  """
+
+  # locate our associated test case in this directory
+  input = Path(__file__).parent / "1902.dot"
+  assert input.exists(), "unexpectedly missing test case"
+
+  # ask Graphviz to process it, which should generate a segfault if this bug
+  # has been reintroduced
+  dot("svg", input)
 
 # root directory of this checkout
 ROOT = Path(__file__).parent.parent.resolve()
@@ -824,14 +843,52 @@ def test_html(src: Path):
   """
 
   # validate the file
-  p = subprocess.Popen(["xmllint", "--nonet", "--noout", "--html", "--valid",
-    src], stderr=subprocess.PIPE, universal_newlines=True)
-  _, stderr = p.communicate()
+  with subprocess.Popen(["xmllint", "--nonet", "--noout", "--html", "--valid",
+                         src], stderr=subprocess.PIPE,
+                        universal_newlines=True) as p:
+    _, stderr = p.communicate()
 
-  # expect it to succeed
-  assert p.returncode == 0
+    # expect it to succeed
+    assert p.returncode == 0
   assert stderr == ""
 
+def test_1855():
+  """
+  SVGs should have a scale with sufficient precision
+  https://gitlab.com/graphviz/graphviz/-/issues/1855
+  """
+
+  # locate our associated test case in this directory
+  src = Path(__file__).parent / "1855.dot"
+  assert src.exists(), "unexpectedly missing test case"
+
+  # run it through Graphviz
+  svg = dot("svg", src)
+
+  # find the graph element
+  root = ET.fromstring(svg)
+  graph = root[0]
+  assert graph.get("class") == "graph", "could not find graph element"
+
+  # extract its `transform` attribute
+  transform = graph.get("transform")
+
+  # this should begin with a scale directive
+  m = re.match(r"scale\((?P<x>\d+(\.\d*)?) (?P<y>\d+(\.\d*))\)", transform)
+  assert m is not None, f"failed to find 'scale' in '{transform}'"
+
+  x = m.group("x")
+  y = m.group("y")
+
+  # the scale should be somewhere in reasonable range of what is expected
+  assert float(x) >= 0.32 and float(x) <= 0.34, "inaccurate x scale"
+  assert float(y) >= 0.32 and float(y) <= 0.34, "inaccurate y scale"
+
+  # two digits of precision are insufficient for this example, so require a
+  # greater number of digits in both scale components
+  assert len(x) > 4, "insufficient precision in x scale"
+  assert len(y) > 4, "insufficient precision in y scale"
+
 @pytest.mark.parametrize("variant", [1, 2])
 @pytest.mark.skipif(shutil.which("gml2gv") is None, reason="gml2gv not available")
 def test_1869(variant: int):
@@ -853,6 +910,24 @@ def test_1869(variant: int):
   assert "style=dashed" in output, "style=dashed not found in DOT output"
   assert "penwidth=2" in output, "penwidth=2 not found in DOT output"
 
+@pytest.mark.xfail() # FIXME
+def test_1879():
+  """https://gitlab.com/graphviz/graphviz/-/issues/1879"""
+
+  # locate our associated test case in this directory
+  input = Path(__file__).parent / "1879.dot"
+  assert input.exists(), "unexpectedly missing test case"
+
+  # process it with DOT
+  stdout = subprocess.check_output(["dot", "-Tsvg", "-o", os.devnull, input],
+                                   cwd=Path(__file__).parent,
+                                   stderr=subprocess.STDOUT,
+                                   universal_newlines=True)
+
+  # check we did not trigger an assertion failure
+  print(stdout)
+  assert re.search(r"\bAssertion\b.*\bfailed\b", stdout) is None
+
 def test_1893():
   """
   an HTML label containing just a ] should work
@@ -863,18 +938,12 @@ def test_1893():
   input = "digraph { 0 [label=<<TABLE><TR><TD>]</TD></TR></TABLE>>] }"
 
   # ask Graphviz to process this
-  p = subprocess.Popen(["dot", "-Tsvg", "-o", os.devnull],
-                       stdin=subprocess.PIPE, universal_newlines=True)
-  p.communicate(input)
-  assert p.returncode == 0
+  dot("svg", source=input)
 
   # we should be able to do the same with an escaped ]
   input = "digraph { 0 [label=<<TABLE><TR><TD>&#93;</TD></TR></TABLE>>] }"
 
-  p = subprocess.Popen(["dot", "-Tsvg", "-o", os.devnull],
-                       stdin=subprocess.PIPE, universal_newlines=True)
-  p.communicate(input)
-  assert p.returncode == 0
+  dot("svg", source=input)
 
 def test_1906():
   """
@@ -888,11 +957,11 @@ def test_1906():
   assert input.exists(), "unexpectedly missing test case"
 
   # use Circo to translate it to DOT
-  p = subprocess.Popen(["dot", "-Kcirco", "-Tgv", "-o", os.devnull, input],
-    stderr=subprocess.PIPE, universal_newlines=True)
-  _, stderr = p.communicate()
+  with subprocess.Popen(["dot", "-Kcirco", "-Tgv", "-o", os.devnull, input],
+                        stderr=subprocess.PIPE, universal_newlines=True) as p:
+    _, stderr = p.communicate()
 
-  assert p.returncode != 0, "graph that generates overflow was accepted"
+    assert p.returncode != 0, "graph that generates overflow was accepted"
 
   assert "area too large" in stderr, "missing/incorrect error message"
 
@@ -907,9 +976,8 @@ def test_1907():
   input = "digraph { A -> B -> C }"
 
   # generate an SVG from this input with twopi
-  p = subprocess.Popen(["twopi", "-Tsvg"], stdin=subprocess.PIPE, stdout=subprocess.PIPE,
+  output = subprocess.check_output(["twopi", "-Tsvg"], input=input,
     universal_newlines=True)
-  output, _ = p.communicate(input)
 
   assert "<title>A&#45;&gt;B</title>" in output, \
     "element title not found in SVG"
@@ -926,11 +994,8 @@ def test_1909():
   graph = Path(__file__).parent / "1909.dot"
 
   # run GVPR with the given input
-  p = subprocess.Popen(["gvpr", "-c", "-f", prog, graph],
-    stdout=subprocess.PIPE, universal_newlines=True)
-  output, _ = p.communicate()
-
-  assert p.returncode == 0, "gvpr failed to process graph"
+  output = subprocess.check_output(["gvpr", "-c", "-f", prog, graph],
+    universal_newlines=True)
 
   # we should have produced this graph without names like "%2" in it
   assert output == "// begin\n" \
@@ -955,8 +1020,7 @@ def test_1910():
   assert c_src.exists(), "missing test case"
 
   # run the test
-  ret, _, _ = run_c(c_src, link=["cgraph", "gvc"])
-  assert ret == 0
+  _, _ = run_c(c_src, link=["cgraph", "gvc"])
 
 def test_1913():
   """
@@ -974,10 +1038,11 @@ def test_1913():
     """
     run Dot with the given input and return its exit status and stderr
     """
-    p = subprocess.Popen(["dot", "-Tsvg", "-o", os.devnull],
-      stdin=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
-    _, stderr = p.communicate(input)
-    return p.returncode, stderr
+    with subprocess.Popen(["dot", "-Tsvg", "-o", os.devnull],
+                          stdin=subprocess.PIPE, stderr=subprocess.PIPE,
+                          universal_newlines=True) as p:
+      _, stderr = p.communicate(input)
+      return p.returncode, remove_xtype_warnings(stderr)
 
   # Graphviz should accept all legal values for this attribute
   for align in ("left", "right", "center"):
@@ -1036,10 +1101,7 @@ def test_1931():
           '}'
 
   # ask Graphviz to process this to dot output
-  p = subprocess.Popen(["dot", "-Txdot"], stdin=subprocess.PIPE,
-    stdout=subprocess.PIPE, universal_newlines=True)
-  xdot, _ = p.communicate(graph)
-  assert p.returncode == 0
+  xdot = dot("xdot", source=graph)
 
   # all new lines in strings should have been preserved
   assert "line 1\nline 2\n" in xdot
@@ -1071,10 +1133,11 @@ def test_1971():
   # run edgepaint with an invalid option, `-rabbit`, that happens to have the
   # same first character as valid options
   args = ["edgepaint", "-rabbit"]
-  p = subprocess.Popen(args, stdin=subprocess.PIPE, universal_newlines=True)
-  p.communicate(input)
+  with subprocess.Popen(args, stdin=subprocess.PIPE,
+                        universal_newlines=True) as p:
+    p.communicate(input)
 
-  assert p.returncode != 0, "edgepaint incorrectly accepted '-rabbit'"
+    assert p.returncode != 0, "edgepaint incorrectly accepted '-rabbit'"
 
 @pytest.mark.xfail(strict=not is_ndebug_defined()) # FIXME
 def test_1990():
@@ -1106,8 +1169,7 @@ def test_2057():
   assert c_src.exists(), "missing test case"
 
   # run the test
-  ret, _, _ = run_c(c_src, link=["gvc"])
-  assert ret == 0
+  _, _ = run_c(c_src, link=["gvc"])
 
 def test_2078():
   """
@@ -1124,11 +1186,12 @@ def test_2078():
           "}"
 
   # run it through Graphviz
-  p = subprocess.Popen(["dot", "-Tcanon", "-o", os.devnull],
-    stdin=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
-  _, stderr = p.communicate(input)
+  with subprocess.Popen(["dot", "-Tcanon", "-o", os.devnull],
+                        stdin=subprocess.PIPE, stderr=subprocess.PIPE,
+                        universal_newlines=True) as p:
+    _, stderr = p.communicate(input)
 
-  assert p.returncode != 0, "layout on subgraph was incorrectly accepted"
+    assert p.returncode != 0, "layout on subgraph was incorrectly accepted"
 
   assert "layout attribute is invalid except on the root graph" in stderr, \
     "expected warning not found"
@@ -1141,12 +1204,12 @@ def test_2078():
           "}"
 
   # ensure this one does not trigger warnings
-  p = subprocess.Popen(["dot", "-Tcanon", "-o", os.devnull],
-    stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
-    universal_newlines=True)
-  stdout, stderr = p.communicate(input)
+  with subprocess.Popen(["dot", "-Tcanon", "-o", os.devnull],
+                        stdin=subprocess.PIPE, stdout=subprocess.PIPE,
+                        stderr=subprocess.PIPE, universal_newlines=True) as p:
+    stdout, stderr = p.communicate(input)
 
-  assert p.returncode == 0, "correct layout use was rejected"
+    assert p.returncode == 0, "correct layout use was rejected"
 
   assert stdout.strip() == "", "unexpected output"
   assert "layout attribute is invalid except on the root graph" not in stderr, \
@@ -1164,7 +1227,7 @@ def test_2082():
 
   # ask Graphviz to process it, which should generate an assertion failure if
   # this bug has been reintroduced
-  subprocess.check_call(["dot", "-Tpng", "-o", os.devnull, input])
+  dot("png", input)
 
 @pytest.mark.xfail(strict=True)
 @pytest.mark.parametrize("html_like_first", (False, True))
@@ -1187,11 +1250,7 @@ def test_2089(html_like_first: bool): # FIXME
             '}'
 
   # normalize the graph
-  p = subprocess.Popen(["dot", "-Tdot"], stdin=subprocess.PIPE,
-                       stdout=subprocess.PIPE, universal_newlines=True)
-  canonical, _ = p.communicate(graph)
-
-  assert p.returncode == 0
+  canonical = dot("dot", source=graph)
 
   assert 'label=foo' in canonical, "non-HTML-like label not found"
   assert "label=<foo>" in canonical, "HTML-like label not found"
@@ -1208,10 +1267,18 @@ def test_2089_2():
   assert c_src.exists(), "missing test case"
 
   # run it
-  ret, stdout, stderr = run_c(c_src, link=["cgraph"])
-  sys.stdout.write(stdout)
-  sys.stderr.write(stderr)
-  assert ret == 0
+  _, _ = run_c(c_src, link=["cgraph"])
+
+@pytest.mark.skipif(shutil.which("dot2gxl") is None,
+                    reason="dot2gxl not available")
+@pytest.mark.xfail(strict=True) # FIXME
+def test_2092():
+  """
+  an empty node ID should not cause a dot2gxl NULL pointer dereference
+  https://gitlab.com/graphviz/graphviz/-/issues/2092
+  """
+  subprocess.run(["dot2gxl", "-d"], input='<node id="">', check=True,
+                 universal_newlines=True)
 
 @pytest.mark.skipif(os.environ.get("build_system") == "msbuild" and
                     os.environ.get("configuration") == "Debug",
@@ -1228,7 +1295,7 @@ def test_2095():
   assert input.exists(), "unexpectedly missing test case"
 
   # ask Graphviz to process it
-  subprocess.check_call(["dot", "-Tpdf", "-o", os.devnull, input])
+  dot("pdf", input)
 
 @pytest.mark.skipif(shutil.which("gv2gml") is None,
                     reason="gv2gml not available")
@@ -1242,11 +1309,11 @@ def test_2131():
   input = "digraph { a -> b; }"
 
   # ask gv2gml what it thinks of this
-  p = subprocess.Popen(["gv2gml"], stdin=subprocess.PIPE,
-                       universal_newlines=True)
-  p.communicate(input)
-
-  assert p.returncode == 0, "gv2gml rejected a basic graph"
+  try:
+    subprocess.run(["gv2gml"], input=input, check=True,
+                   universal_newlines=True)
+  except subprocess.CalledProcessError as e:
+    raise RuntimeError("gv2gml rejected a basic graph") from e
 
 @pytest.mark.skipif(shutil.which("gvpr") is None,
                     reason="gvpr not available")
@@ -1286,6 +1353,167 @@ def test_2138(examine: str):
     assert "// tok[7]    >>012<<   should NOT include trailing spaces or "     \
       "junk chars" in result, "token 012 not found or has trailing garbage"
 
+@pytest.mark.xfail(strict=True) # FIXME
+def test_2179():
+  """
+  processing a label with an empty line should not yield a warning
+  https://gitlab.com/graphviz/graphviz/-/issues/2179
+  """
+
+  # a graph containing a label with an empty line
+  input = 'digraph "" {\n' \
+          '  0 -> 1 [fontname="Lato",label=<<br/>1>]\n' \
+          '}'
+
+  # run a graph with an empty label through Graphviz
+  with subprocess.Popen(["dot", "-Tsvg", "-o", os.devnull],
+                        stdin=subprocess.PIPE, stderr=subprocess.PIPE,
+                        universal_newlines=True) as p:
+    _, stderr = p.communicate(input)
+
+    assert p.returncode == 0
+
+  assert "Warning: no hard-coded metrics for" not in stderr, \
+    "incorrect warning triggered"
+
+# FIXME: this has been root caused to 84b2983edf458098bb6233368904265c92da4e65
+# but a fix has not yet been devised
+@pytest.mark.xfail()
+def test_2185_1():
+  """
+  GVPR should deal with strings correctly
+  https://gitlab.com/graphviz/graphviz/-/issues/2185
+  """
+
+  # find our collocated GVPR program
+  script = Path(__file__).parent / "2185.gvpr"
+  assert script.exists(), "missing test case"
+
+  # run this with NUL input, checking output is valid UTF-8
+  _ = subprocess.check_output(["gvpr", "-f", script],
+                              stdin=subprocess.DEVNULL,
+                              universal_newlines=True)
+
+@pytest.mark.xfail()
+def test_2185_2():
+  """
+  GVPR should deal with strings correctly
+  https://gitlab.com/graphviz/graphviz/-/issues/2185
+  """
+
+  # find our collocated GVPR program
+  script = Path(__file__).parent / "2185.gvpr"
+  assert script.exists(), "missing test case"
+
+  # run this with NUL input
+  out = subprocess.check_output(["gvpr", "-f", script],
+                                stdin=subprocess.DEVNULL)
+
+  # decode output in a separate step to gracefully cope with garbage unicode
+  out = out.decode("utf-8", "replace")
+
+  # deal with Windows eccentricities
+  eol = "\r\n" if platform.system() == "Windows" else "\n"
+  expected = f"one two three{eol}"
+
+  # check the first line is as expected
+  assert out.startswith(expected), "incorrect GVPR interpretation"
+
+# FIXME: like `test_2185_1`, it is believed that
+# 84b2983edf458098bb6233368904265c92da4e65 is the root cause of this
+@pytest.mark.xfail()
+def test_2185_3():
+  """
+  GVPR should deal with strings correctly
+  https://gitlab.com/graphviz/graphviz/-/issues/2185
+  """
+
+  # find our collocated GVPR program
+  script = Path(__file__).parent / "2185.gvpr"
+  assert script.exists(), "missing test case"
+
+  # run this with NUL input
+  out = subprocess.check_output(["gvpr", "-f", script],
+                                stdin=subprocess.DEVNULL)
+
+  # decode output in a separate step to gracefully cope with garbage unicode
+  out = out.decode("utf-8", "replace")
+
+  # deal with Windows eccentricities
+  eol = "\r\n" if platform.system() == "Windows" else "\n"
+  expected = f"one two three{eol}one  five three{eol}"
+
+  # check the first two lines are as expected
+  assert out.startswith(expected), "incorrect GVPR interpretation"
+
+@pytest.mark.xfail() # FIXME
+def test_2185_4():
+  """
+  GVPR should deal with strings correctly
+  https://gitlab.com/graphviz/graphviz/-/issues/2185
+  """
+
+  # find our collocated GVPR program
+  script = Path(__file__).parent / "2185.gvpr"
+  assert script.exists(), "missing test case"
+
+  # run this with NUL input
+  out = subprocess.check_output(["gvpr", "-f", script],
+                                stdin=subprocess.DEVNULL)
+
+  # decode output in a separate step to gracefully cope with garbage unicode
+  out = out.decode("utf-8", "replace")
+
+  # deal with Windows eccentricities
+  eol = "\r\n" if platform.system() == "Windows" else "\n"
+  expected = f"one two three{eol}one  five three{eol}99{eol}"
+
+  # check the first three lines are as expected
+  assert out.startswith(expected), "incorrect GVPR interpretation"
+
+@pytest.mark.xfail() # FIXME
+def test_2185_5():
+  """
+  GVPR should deal with strings correctly
+  https://gitlab.com/graphviz/graphviz/-/issues/2185
+  """
+
+  # find our collocated GVPR program
+  script = Path(__file__).parent / "2185.gvpr"
+  assert script.exists(), "missing test case"
+
+  # run this with NUL input
+  out = subprocess.check_output(["gvpr", "-f", script],
+                                stdin=subprocess.DEVNULL)
+
+  # decode output in a separate step to gracefully cope with garbage unicode
+  out = out.decode("utf-8", "replace")
+
+  # deal with Windows eccentricities
+  eol = "\r\n" if platform.system() == "Windows" else "\n"
+  expected = f"one two three{eol}one  five three{eol}99{eol}Constant{eol}"
+
+  # check the first four lines are as expected
+  assert out.startswith(expected), "incorrect GVPR interpretation"
+
+@pytest.mark.xfail(strict=True) # FIXME
+def test_2193():
+  """
+  the canonical format should be stable
+  https://gitlab.com/graphviz/graphviz/-/issues/2193
+  """
+
+  # find our collocated test case
+  input = Path(__file__).parent / "2193.dot"
+  assert input.exists(), "unexpectedly missing test case"
+
+  # derive the initial canonicalization
+  canonical = dot("canon", input)
+
+  # now canonicalize this again to see if it changes
+  new = dot("canon", source=canonical)
+  assert canonical == new, "canonical translation is not stable"
+
 def test_package_version():
   """
   The graphviz_version.h header should define a non-empty PACKAGE_VERSION
@@ -1301,8 +1529,7 @@ def test_package_version():
   assert c_src.exists(), "missing test case"
 
   # run the test
-  ret, _, _ = run_c(c_src)
-  assert ret == 0
+  _, _ = run_c(c_src)
 
 def test_user_shapes():
   """
@@ -1334,7 +1561,7 @@ def test_xdot_json():
 
   # ask our C helper to process this
   try:
-    ret, output, err = run_c(c_src, input=input, link=["xdot"])
+    output, err = run_c(c_src, input=input, link=["xdot"])
   except subprocess.CalledProcessError:
     # FIXME: Remove this try-catch when
     # https://gitlab.com/graphviz/graphviz/-/issues/1777 is fixed
@@ -1342,7 +1569,6 @@ def test_xdot_json():
       pytest.skip("Windows MSBuild release does not contain any header "
                   "files (#1777)")
     raise
-  assert ret == 0
   assert err == ""
 
   if os.getenv("build_system") == "msbuild":
@@ -1350,12 +1576,12 @@ def test_xdot_json():
                   "Graphviz header. Remove the above try-catch? (#1777)")
 
   # confirm the output was what we expected
-  assert output == '[\n'                                                   \
-                   '{c : "#fffffe00"},\n'                                  \
-                   '{C : "#ffffff"},\n'                                    \
-                   '{P : [0.000000,0.000000,0.000000,36.000000,54.000000,' \
-                     '36.000000,54.000000,0.000000]}\n'                    \
-                   ']\n'
+  data = json.loads(output)
+  assert data == [
+                  {"c" : "#fffffe00"},
+                  {"C" : "#ffffff"},
+                  {"P" : [0.0, 0.0, 0.0, 36.0, 54.0, 36.0, 54.0, 0.0]}
+                 ]
 
 # find all .vcxproj files
 VCXPROJS: List[Path] = []
@@ -1450,7 +1676,41 @@ def test_gvmap_fclose():
           '}'
 
   # pass this through gvmap
-  p = subprocess.Popen(["gvmap"], stdin=subprocess.PIPE)
-  p.communicate(input.encode("utf-8"))
+  subprocess.run(["gvmap"], input=input.encode("utf-8"), check=True)
 
-  assert p.returncode == 0
+def test_bitarray():
+  """run the bitarray unit tests"""
+
+  # locate the bitarray unit tests
+  src = Path(__file__).parent.resolve() / "../lib/cgraph/test_bitarray.c"
+  assert src.exists()
+
+  # locate lib directory that needs to be in the include path
+  lib = Path(__file__).parent.resolve() / "../lib"
+
+  # extra C flags this compilation needs
+  cflags = ["-I", lib]
+  if platform.system() != "Windows":
+    cflags += ["-std=gnu99", "-Wall", "-Wextra", "-Werror"]
+
+  _, _ = run_c(src, cflags=cflags)
+
+@pytest.mark.skipif(shutil.which("gvpr") is None, reason="gvpr not available")
+def test_gvpr_usage():
+  """
+  gvpr usage information should be included when erroring on a malformed command
+  """
+
+  # create a temporary directory, under which we know no files will exist
+  with tempfile.TemporaryDirectory() as tmp:
+
+    # ask GVPR to process a non-existent file
+    with subprocess.Popen(["gvpr", "-f", "nofile"], stderr=subprocess.PIPE,
+                          cwd=tmp, universal_newlines=True) as p:
+      _, stderr = p.communicate()
+
+      assert p.returncode != 0, "GVPR accepted a non-existent file"
+
+  # the stderr output should have contained full usage instructions
+  assert "-o <ofile> - write output to <ofile>; stdout by default" in stderr, \
+    "truncated or malformed GVPR usage information"
